
/* This headerfile should ONLY be used by the device-dependent modules */
/* "u_pl_xxx.c", NEVER by a user program                               */

#define PLOT_STAT_OK             0
#define PLOT_STAT_NOMOREHANDLES  1
#define PLOT_STAT_NOMEMORY       2
#define PLOT_STAT_NOSUCHHANDLE   3
#define PLOT_STAT_NOMOREDEV      4
#define PLOT_STAT_NODISPLAY      5
#define PLOT_STAT_NOSUCHDEV      6
#define PLOT_STAT_OPENERROR      7
#define PLOT_STAT_NOSYMBOLS      8
#define PLOT_STAT_NODRIVER       9

#define MAXHANDLES 20

#define UNIX
#ifdef VMS
typedef char symbolcoordtyp;
#else
typedef signed char symbolcoordtyp;
#endif

typedef struct symboltyp{
   unsigned char vec;
   symbolcoordtyp *x1,*x2,*y1,*y2;
}symboltyp;

/* Forward declarationof handletyp, we need handleptr to be declared */
/* to use it in the declaration of handletyp. Gnu C complains about  */
/* "useless keyword in empty declaration", don't care about...       */

/* handletyp contains all information about a plot device handle, and */
/* pointers to the functions you need for working with  the device    */
#ifdef UNIX
/* the C compiler on Ultrix does not support the declaration of */
/* function pointers with argument lists                        */ 
typedef struct handletyp {
   int xvmin, xvmax, yvmin, yvmax, clip;     
   double xwmin, xwmax, ywmin, ywmax, xfact, xoffs, yfact, yoffs;
   double akt_x, akt_y, symbol_factor, cm_per_unit;
   char name[80];
   int devtype;
   unsigned long capabilities;
   int (* ploth_modify)();
   int (* ploth_getdevice)();
   void (* ploth_line)();
   void (* ploth_point)();
   void (* ploth_fillpolygon)();
   void (* ploth_polyline)();
   int (* ploth_setfg)();
   int (* ploth_getfg)();
   int (* ploth_setbg)();
   int (* ploth_getbg)();
   void (* ploth_settextalign)();
   void (* ploth_writetext)();
   void (* ploth_erasetext)();
   void (* ploth_graphmode)();
   void (* ploth_clear)();
   void (* ploth_textmode)();
   void (* ploth_close)();
   int  (* ploth_setmode)();
   int  (* ploth_getmode)();
   int (* ploth_getpointer)();
   int (* ploth_checkevent)();
   int (* ploth_get_any_event)();
   int (* ploth_check_any_event)();
   void (* ploth_rectangle)();
   void (* ploth_polysymbol)();
   void (* ploth_do_special)();
} handletyp;
#else
typedef struct handletyp {
   /* variables for viewport (device coordinates) */
   int xvmin, xvmax, yvmin, yvmax, clip;
   /* variables for window (user coordinates) */
   double xwmin, xwmax, ywmin, ywmax, xfact, xoffs, yfact, yoffs;
   /* actual output coordinates and size of symbols */
   double akt_x, akt_y, symbol_factor, cm_per_unit;
   /* device name: filename for hardcopy-devices, titlebar for windows */
   char name[80];
   /* Type of device */
   int devtype;
   /* Bitmask for capabilities of the device, see u_pl_def.h for */
   /* the meaning of the bits.                                   */ 
   unsigned long capabilities;
   /* Modify the device, if possible: size, position, name */
   int (* ploth_modify)(int nh, handleptr h, int sizex, int sizey,
  		        int posx, int posy, char name[]);
   /* Get device size, returns device type */
   int (* ploth_getdevice)(int handle, int *sizex, int *sizey);
   /** drawing - device coordinates */
   /* draw a line from (X0,Y0) to (X1,Y1) */
   void (* ploth_line)(int handle, int X0, int Y0, int X1, int Y1);
   /* plot a point at (X,Y) */
   void (* ploth_point)(int handle, int X, int Y);
   /* draw a polyline with pts points (X,Y) */
   void (* ploth_polyline)(int handle, int pts, int X[], int Y[]);
   void (* ploth_fillpolygon)(int handle, int pts, int X[], int Y[]);
   /* Set foreground color, returns the resulting color */
   int (* ploth_setfg)(int handle, int fg);
   /* Return actual foreground color */
   int (* ploth_getfg)(int handle);
   /* Set background color, returns the resulting color */
   int (* ploth_setbg)(int handle, int bg);
   /* Return actual background color */
   int (* ploth_getbg)(int handle);
   /* Set text alignment */
   void (* ploth_settextalign)(int handle, int xalign, int yalign);
   /* Write txt at (X,Y) using the actual alignment */
   void (* ploth_writetext)(int handle, int x, int y, char txt[]);
   /* Try to erase txt at (X,Y) using the actual alignment, */
   /* ignored on hardcopy devices                           */
   void (* ploth_erasetext)(int handle, int x, int y, char txt[]);
   /* Switch device to graphic mode */
   void (* ploth_graphmode)(int handle);
   /* Clear device: Clear screen or form feed */
   void (* ploth_clear)(int handle);
   /* Switch device to text mode */
   void (* ploth_textmode)(int handle);
   /* Close the device: Close file, destroy window, free allocated memory */
   void (* ploth_close)(int handle);
   /* Set output mode, e.g. AND, OR, XOR,..., if supported. */
   /* returns the resulting output mode                     */
   int  (* ploth_setmode)(int handle, int mode);
   /* returns the actual output mode */
   int  (* ploth_getmode)(int handle);
   /************ the following 6 routines are optional. **************/
   /** if they are not implemented, u_plot sets the function pointer */
   /** to NULL. Each routine calling one of them should check for    */
   /** NULL before.                                                  */
   /******************************************************************/
   /* Wait for keypress or mouse button, and return coordinates of */
   /* pointer and code for key/button                              */
   int (* ploth_getpointer)(int handle, int *x, int *y);
   /* Check if a key or mouse button was pressed, and return coordinates */
   /* of pointer and code for key/button                                 */
   int (* ploth_checkevent)(int handle, int *x, int *y);
   /* Next two: Same as _getpointer and _checkevent, but they work on any */
   /* handle which uses the same device driver. These routines are        */
   /* intended for window systems, if you have more than one plotwindow   */
   int (* ploth_get_any_event)(int handle, int *at_handle, int *x, int *y);
   int (* ploth_check_any_event)(int handle, int *at_handle, int *x, int *y);
   /* Draw a rectangle, fill it with the color "fillcolor". Filling may */
   /* be ignored if the device does not support it.                     */
   void (* ploth_rectangle)(int handle, int x1, int y1, int x2, int y2, 
                            int fillcolor);
   /* Plot a sequence of lines (symbol) for each point in (X,Y)  */
   /* if not implemented, u_plot does this using single lines.   */
   /* On X-Window systems, use of this function can accelerate   */
   /* output by some orders of magnitude                         */
   void (* ploth_polysymbol)(int handle, int pts, symboltyp sym, 
                             int x[], int y[]);
   /* Write a string transparently to the device, if appropriate */
   /* The device driver may ignore it. This routine is optional  */
   void (* ploth_do_special)(int handle, char str[]);
} handletyp;

#endif
/*typedef struct handletyp;*/    
typedef struct handletyp *handleptr;
