
#include "u_pl_loc.h"
#include "u_plot.h"
#include "v_inp.h"

#include <stdio.h>
#include <string.h>

#define MAXCOLOR      8
#define DEFCOLOR      6
#define DEFBACKGROUND 0
#define DEFMODE       PLOT_MODE_OVER

#define ESC 27

static int devsizex, devsizey;
static int regis_is_open = FALSE;
static int background, foreground, mode, text_xalign, text_yalign;


int plotd_modify_regis(int nh, handletyp *h, int sizex, int sizey,
  		       int posx, int posy, char name[]){
  return(PLOT_STAT_OK); }

static int plotd_setmode_regis(int handle, int newmode){
   if (newmode==PLOT_USEDEFAULT) newmode = DEFMODE;
   switch (newmode){
       case PLOT_MODE_REPLACE : printf("W(R)\n");break; /* replace    */
       case PLOT_MODE_ERASE   : printf("W(E)\n");break; /* erase      */
       case PLOT_MODE_OVER    : printf("W(V)\n");break; /* Overlay    */
       case PLOT_MODE_COMP    : printf("W(C)\n");break; /* Complement */
       default : return(mode);
   }
   mode = newmode; return(mode);
}

static int plotd_getmode_regis(int handle){
   return(mode); }

static void plotd_settextalign_regis(int handle, int xalign, int yalign){
   text_xalign = xalign;
   text_yalign = yalign;
}

static int plotd_getdevice_regis(int handle, int *sizex, int *sizey){
   *sizex = devsizex; *sizey = devsizey;
   return(PLOT_DEV_REGIS); }

static void plotd_line_regis(int handle, int X0, int Y0, int X1, int Y1){
    printf("P[%1d,%1d]V[%1d,%1d]",X0,devsizey-Y0,X1,devsizey-Y1);}

static void plotd_point_regis(int handle, int X, int Y){
    printf("P[%1d,%1d]V[%1d,%1d]",X,devsizey-Y,X,devsizey-Y);}

static void plotd_polyline_regis(int handle, int pts, int X[], int Y[]){
   int i;
   printf("P[%1d,%1d]",X[0],devsizey-Y[0]);
   for (i=1; i<pts; i++) printf("V[%1d,%1d]",X[i],devsizey-Y[i]);
}

static char regiscol[9] = "DBRMGCYW";       /* names of RGB colors for REGIS */

static int plotd_setfg_regis(int handle, int fg){
   if (fg==PLOT_USEDEFAULT) fg = DEFCOLOR;
   fg = fg % MAXCOLOR;
   if (fg==background) fg=DEFCOLOR;
   printf("W(I(%c))\n",regiscol[fg]);
   foreground = fg;
   return(foreground);
}

static int plotd_getfg_regis(int handle){
   return(foreground);
}

static int plotd_setbg_regis(int handle, int bg){
   if (bg==PLOT_USEDEFAULT) bg = DEFBACKGROUND;
   bg = bg % MAXCOLOR;
   printf("W(I(%c))\n",regiscol[bg]);
   background = bg;
   return(background);
}


static int plotd_getbg_regis(int handle){
   return(background);
}


static void TextExtent(char txt[], int *sizex, int *sizey){
    *sizex = 8 * strlen(txt);
    *sizey = 10;
}

static void plotd_writetext_regis(int handle, int x, int y, char txt[]){
    int sizex, sizey;
    TextExtent(txt,&sizex,&sizey);
    if (text_xalign==PLOT_RIGHTTEXT)  x = x - sizex;
    if (text_xalign==PLOT_CENTERTEXT) x = x - sizex/2;
    if (text_yalign==PLOT_CENTERTEXT) y = y - sizey/2;
    if (text_yalign==PLOT_TOPTEXT)    y = y - sizey;
    printf("P[%1d,%1d]T22\"%s\"\n",x,devsizey-y,txt);}

static void plotd_erasetext_regis(int handle, int x, int y, char txt[]){
   int mode;
   mode = plotd_getmode_regis(handle);
   plotd_setmode_regis(handle,PLOT_MODE_ERASE);
   plotd_writetext_regis(handle,x,y,txt);
   plotd_setmode_regis(handle,mode);
}

static void plotd_graphmode_regis(int handle){
   printf("%cP0p",ESC);
   text_xalign = PLOT_LEFTTEXT;
   text_yalign = PLOT_BOTTOMTEXT;
}

static void plotd_clear_regis(int handle){
   printf("S(E)\n");}

static void plotd_textmode_regis(int handle){
   printf("T'x'%c\\\n",ESC); }

static void plotd_close_regis(int handle){
/*   plotd_clear_regis(handle);
   plotd_textmode_regis(handle); */
   regis_is_open = FALSE;
}

static int plotd_getpointer_regis(int handle, int *x, int *y){
/* set cursor type "S(C1(In))" not supported by DECterm */
   char *coord, trm, instr[30];
   double out[16];
   *x = 0; *y = 0;
   printf("R(I0)R(P(I))");       /* "one shot" mode, request report */
#ifdef UNIX
   getstr(instr);                  /* dummy read necessary */
   getstr(instr);
#else
   gets(instr);                  /* dummy read necessary */
   gets(instr);
#endif
   if ((instr[0]==ESC) && (strchr(instr,'~'))){  /* Mouse button: ESC [ 2 4 n ~ */
      coord = (strchr(instr,'~')+1);
      switch(instr[4]){
         case '1' : trm = PLOT_TRM_MB_LEFT;   break;
         case '3' : trm = PLOT_TRM_MB_CENTER; break;
         case '5' : trm = PLOT_TRM_MB_RIGHT;  break;
         default  : trm = ' ';}     /* Unknown... */
   } else {                       /* other terminator: one char */
      coord = (instr + 1);
      trm = instr[0];
   }
   if (trans(coord,out)==2){
      *x = out[0]; *y = devsizey - out[1];}
   return(trm);
}


int plotd_init_regis(int nh, handletyp *h, int sizex, int sizey,
		     int posx, int posy, char name[]){
   if (regis_is_open) return(PLOT_STAT_NOMOREDEV);
   regis_is_open = TRUE;
   devsizex = 799;  devsizey = 479;
   h->ploth_modify     =   plotd_modify_regis;
   h->ploth_getdevice  =   plotd_getdevice_regis;
   h->ploth_line       =   plotd_line_regis;
   h->ploth_point      =   plotd_point_regis;
   h->ploth_polyline   =   plotd_polyline_regis;
   h->ploth_setfg      =   plotd_setfg_regis;
   h->ploth_getfg      =   plotd_getfg_regis;
   h->ploth_setbg      =   plotd_setbg_regis;
   h->ploth_getbg      =   plotd_getbg_regis;
   h->ploth_writetext  =   plotd_writetext_regis;
   h->ploth_erasetext  =   plotd_erasetext_regis;
   h->ploth_graphmode  =   plotd_graphmode_regis;
   h->ploth_clear      =   plotd_clear_regis;
   h->ploth_textmode   =   plotd_textmode_regis;
   h->ploth_close      =   plotd_close_regis;
   h->ploth_setmode    =   plotd_setmode_regis;
   h->ploth_getmode    =   plotd_getmode_regis;
   h->ploth_getpointer =   plotd_getpointer_regis;
   h->ploth_settextalign = plotd_settextalign_regis;
   return(PLOT_STAT_OK);
}
