
#include "u_pl_def.h"
#include "u_pl_loc.h"
#include "v_inp.h"

#include <stdio.h>
#include <string.h>

#define MAXCOLOR 8
#define DEFCOLOR 6
#define DEFBACKGROUND 0
#define DEFMODE       PLOT_MODE_OVER

#define ETX   3
#define FF   12
#define CR   13
#define CAN  24
#define SUB  26
#define ESC  27
#define FS   28
#define GS   29
#define US   31

#define CHAR_WIDTH  14
#define CHAR_HEIGHT 14
#define CHAR_OFFSET  7

/* we allow only one handle on the terminal */
static int tek_is_open = 0;
static int mode = DEFMODE;
static int foreground = DEFCOLOR;
static int background = DEFBACKGROUND;
static int text_xalign, text_yalign;

/* There is nothing to modify */
static int plotd_modify_tek(int nh, handleptr h, int sizex, int sizey,
  		     int posx, int posy, char name[]){return(PLOT_STAT_OK);}

/* size is constant */
static int plotd_getdevice_tek(int handle, int *sizex, int *sizey){
   *sizex = 1022; *sizey = 780; return(PLOT_DEV_TEK); }

static void plotd_settextalign_tek(int handle, int xalign, int yalign){
   text_xalign = xalign;
   text_yalign = yalign;
}

static void WriteX(int X){
  char HighX, LowX;
  HighX = ((X / 32) + 32);
  LowX  = ((X % 32) + 64);
  printf("%c%c",HighX,LowX);
}

static void WriteY(int Y){
  char HighY, LowY;
  HighY = ((Y / 32) + 32);
  LowY  = ((Y % 32) + 96);
  printf("%c%c",HighY,LowY);
}

static void plotd_point_tek(int handle, int X, int Y){
   printf("\n%c",FS); WriteY(Y); WriteX(X);
}

static void plotd_line_tek(int handle, int X0, int Y0, int X1, int Y1){
   printf("\n%c",GS);
   WriteY(Y0); WriteX(X0);
   WriteY(Y1); WriteX(X1);
}


static void plotd_polyline_tek(int handle, int pts, int X[], int Y[]){
   int i;
   printf("\n%c",GS);
   for (i=0; i<pts; i++)  {
      WriteY(Y[i]);  WriteX(X[i]);
      if (((i+1) % 5)==0) {
         printf("\n%c",GS); WriteY(Y[i]);  WriteX(X[i]); }
}}

static int plotd_setfg_tek(int handle, int fg){
   if (fg==PLOT_USEDEFAULT) fg = DEFCOLOR;
   fg = fg % MAXCOLOR;
   if (fg==background) fg=DEFCOLOR;
   printf("%c[3%1dm",ESC,fg);
   foreground = fg;
   return(foreground);
}

static int plotd_getfg_tek(int handle){
   return(foreground);}

static int plotd_setbg_tek(int handle, int bg){
   if (bg==PLOT_USEDEFAULT) bg = DEFBACKGROUND;
   bg = bg % MAXCOLOR;
   printf("%c[4%1dm",ESC,bg);
   background = bg;
   return(background);
}

static int plotd_getbg_tek(int handle){
   return(background);}

static int plotd_setmode_tek(int handle, int newmode){
   if (newmode==PLOT_USEDEFAULT) newmode=DEFMODE;
   switch (newmode){
           case PLOT_MODE_REPLACE  : printf("%c/1d",ESC); break;
           case PLOT_MODE_COMP     : printf("%c/0d",ESC); break;
           case PLOT_MODE_OVER     : printf("%c/0d",ESC); break;
           case PLOT_MODE_ERASE    : printf("%c/2d",ESC); break;
           default : return(mode);
	  }
   mode = newmode;
   return(mode);
}

static int plotd_getmode_tek(int handle){
   return(mode); }

static void plotd_writetext_tek(int handle, int x, int y, char txt[]){
   printf("\n%c",GS);
   if (text_xalign==PLOT_RIGHTTEXT)  x = x - CHAR_WIDTH * strlen(txt);
   if (text_xalign==PLOT_CENTERTEXT) x = x - (CHAR_WIDTH * strlen(txt)) / 2;
   if (text_yalign==PLOT_CENTERTEXT) y = y - CHAR_HEIGHT / 2;
   if (text_yalign==PLOT_TOPTEXT)    y = y - CHAR_HEIGHT;
   y = y - CHAR_OFFSET;
   if (x<0) x = 0;
   if (y<0) y = 0;
   WriteY(y); WriteX(x);
   printf("%c%s\n",US,txt);
}

static void plotd_erasetext_tek(int handle, int x, int y, char txt[]){
   int savemode;
   savemode = plotd_getmode_tek(handle);
   plotd_setmode_tek(handle,PLOT_MODE_ERASE);
   plotd_writetext_tek(handle,x,y,txt);
   plotd_setmode_tek(handle,savemode);
}


static void plotd_graphmode_tek(int handle){
   printf("%c",CAN);
   text_xalign = PLOT_LEFTTEXT; text_yalign = PLOT_BOTTOMTEXT;
}

static void plotd_clear_tek(int handle){
   plotd_line_tek(handle,10,10,11,11);
   printf("%c%c",ESC,FF);
}

static void plotd_textmode_tek(int handle){
   printf("%c",CAN); }

static void plotd_close_tek(int handle){
   tek_is_open = FALSE;
/*   plotd_textmode_tek(handle); */
}

static int plotd_getpointer_tek(int handle, int *x, int *y){
    char cha[5];
    printf("%c%c",ESC,SUB);
#ifdef UNIX
    getstr(cha);
#else
    getc(cha);
#endif
    *x = (cha[2] % 32)+(cha[1] % 32) * 32;
    *y = (cha[4] % 32)+(cha[3] % 32) * 32;
    return cha[0];
}

static int plotd_checkevent_tek(int handle, int *x, int *y){
    char cha[5];
    if (! KeyPressed()) return PLOT_TRM_ABORT;
    printf("%c%c",ESC,SUB);
    getstr(cha);
    *x = (cha[2] % 32)+(cha[1] % 32) * 32;
    *y = (cha[4] % 32)+(cha[3] % 32) * 32;
    return cha[0];
}

int plotd_init_tek(int nh, handletyp *h, int sizex, int sizey,
		     int posx, int posy, char name[]){
   if (tek_is_open) return(PLOT_STAT_NOMOREDEV);
   tek_is_open = TRUE;
   h->ploth_modify     =   plotd_modify_tek;
   h->ploth_getdevice  =   plotd_getdevice_tek;
   h->ploth_line       =   plotd_line_tek;
   h->ploth_point      =   plotd_point_tek;
   h->ploth_polyline   =   plotd_polyline_tek;
   h->ploth_setfg      =   plotd_setfg_tek;
   h->ploth_getfg      =   plotd_getfg_tek;
   h->ploth_setbg      =   plotd_setbg_tek;
   h->ploth_getbg      =   plotd_getbg_tek;
   h->ploth_writetext  =   plotd_writetext_tek;
   h->ploth_erasetext  =   plotd_erasetext_tek;
   h->ploth_graphmode  =   plotd_graphmode_tek;
   h->ploth_clear      =   plotd_clear_tek;
   h->ploth_textmode   =   plotd_textmode_tek;
   h->ploth_close      =   plotd_close_tek;
   h->ploth_setmode    =   plotd_setmode_tek;
   h->ploth_getmode    =   plotd_getmode_tek;
   h->ploth_getpointer =   plotd_getpointer_tek;
   h->ploth_checkevent =   plotd_checkevent_tek;
   h->ploth_settextalign = plotd_settextalign_tek;
   return(PLOT_STAT_OK);
}
