#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>            /* string routines             */
#include <math.h>              /* mathematical routines       */
#include <errno.h>
#include <curses.h>
#include <unistd.h>

#include "v_inp.h"
#define UNIX

#ifdef UNIX
#define PATHSEP '/'
#define DIRCMD "ls"
#endif

#ifdef NTUNIX
#include <curses.h>
#define PATHSEP '/'
#define DIRCMD "cmd /c start cmd /k ls "
#endif

#ifdef VMS
#include <smgdef.h>
#include <smgtrmptr.h>
#include <smg$routines.h>
#include <lib$routines.h>
#include <signal.h>
#include <unixlib.h>
#include <unixio.h>
#define PATHSEP ']'
#define DIRCMD "dir"
#endif

#ifdef MSDOS
#define PATHSEP '\\'
#endif

#ifdef BORLANDC
#include <conio.h>
#define DIRCMD "command /c dir"
#endif

#ifdef GNUPC
#include <pc.h>
char *strerror(int code);
#define DIRCMD "dir"
#endif

#define plus  1.0
#define minus  -1.0

/* ASCII Codes, define after all #include's */
#define NUL   0
#define TAB   9
#define LF   10
#define CR   13
#define ESC  27

#ifdef VMS
   static unsigned keyb_id;
#endif
   char new[80];
void get_string(char*);
void fget_string(FILE*,char*);

void newline(void){
#ifdef UNIX
  /* addch('0'); */
  printf("%c%c",CR,LF); 
#endif
#ifndef UNIX
  printf("\n");
#endif
}

char *mystrerror(int code){
   char *ch;
   ch = strerror(code);
   if (ch == NULL) return " Unknown error ";
   else return ch;
}

static int v_inp_initialized = FALSE;

void init_v_inp(){
    if (v_inp_initialized) return;
    v_inp_initialized = TRUE;
#ifdef VMS
    smg$create_virtual_keyboard(&keyb_id);
#endif
#ifdef UNIX
    initscr();
    /*    immedok(stdscr,TRUE); */
    refresh();
#endif
}

void clrscr(){
  printf("%c[2J%c[1;1H\n",ESC,ESC);
}

#ifdef MSDOS
#define cuserid(x) "Unknown"
#endif
#define cuserid(x) "Unknown"
char *proc_username(){
   return (cuserid(NULL));
}


void taste_warten(FILE *volterm,int wait){
#ifdef UNIX
  refresh();
#endif
  if (!wait) return;
  fprintf(volterm,"\t Press any key to continue \n");
  read_key();
}

double real_lesen(FILE *volterm){
   char instr[80];
   double out[16];
   do{
      fget_string(volterm,instr);
      if (trans(instr,out)) return(out[0]);
   } while(1);
}

long integer_lesen(FILE *volterm){
    return(real_lesen(volterm));
}

double real_default(FILE *volterm,double def){
  char instr[80];
  double out[16];
  fget_string(volterm,instr);
  if (trans(instr,out)) return(out[0]);
  else return(def);
}

long integer_default(FILE *volterm,long def){
  char instr[80];
  double out[16];
  fget_string(volterm,instr);
  if (trans(instr,out)) return(floor(out[0]+0.1));
  else return(def);
  }

char *string_default(FILE *volterm,char def[]){
   fget_string(volterm,new);
   if (strlen(new)==0) strcpy(new,def);
   return(new);
}

char *readlinevef(char str[], int max, FILE *inf){
/* this is a combination of fgets() and pascal's readln(): We read up to */
/* max characters into str[] and skip the rest of the line,              */
/* Trailing CR and/or LF are deleted, TABs are converted to the          */
/* appropriate number of Blanks       */
   char *p, dummy[80], tmp[9];
   int i, j;
   p = fgets(str,max,inf);
   if (p==NULL) return NULL;
   str[max-1] = 0;
   p = strchr(str,LF);
   if (p != NULL) *p=0;
   else while (p==NULL){
      p = fgets(dummy,80,inf);
      if (p==NULL) break;  /* EOF */ 
      p = strchr(dummy,LF);
   }
   p = strchr(str,CR); if (p != NULL) *p=0;
   p = str + strlen(str)-1; while((*p==32) && (p > str)){ *p=0; p--; }
   p = strchr(str,TAB);
   while ((p = strchr(str,TAB)) != NULL){
     i = (int)(p-str); j = (8 - (i % 8));
     memset(tmp,' ',8); tmp[j] = '\0';
     strerase(str,i,1);
     insert(tmp,str,i);     
   }
   return str;
}

void message(FILE *volterm,char msg[])
{
   fprintf(volterm," %s \n",msg);
   taste_warten(volterm,TRUE);
}

char consingle()
{
   char instr[80];
   get_string(instr);
   if (instr[0]) return(instr[0]); else return(CR);
}

int insert(sou,tar,i)
  char sou[],tar[];
  int i;
  { char tmp[200];
    tmp[0] = 0;
    strcpy(tmp,tar);
    tmp[i]=0;
    strcat(tmp,sou);
    strcat(tmp,&tar[i]);
    strcpy(tar,tmp);
    return strlen(tmp);
  }


int strerase(char str[], int index, int count)
{
   str[index] = 0;
   strcat(str, str+index+count);
   return(strlen(str));
}

void directory(char wildcard[]){
   char cmd[80];
   sprintf(cmd,"%s %s",DIRCMD,wildcard);
#ifdef UNIX
   reset_shell_mode();
#endif
   /* #ifdef NTUNIX
      message("Directory not implemented in NT-Version, please use the explorer!"); 
 #else */
   system(cmd);
   /* #endif */
#ifdef UNIX
   reset_prog_mode();
#endif
   newline();
   newline();
}

void upstring(char s[]){
   while (s[0]){
      s[0]=toupper(s[0]);
      s++;
   }
}

int change_dir(char newdir[]){
   if (strlen(newdir)==0) return TRUE;
   if (chdir(newdir)==0) return TRUE;
   newline();
   printf("\tError changing Directory to %s",newdir);
   newline();
   printf("\t%s",mystrerror(errno));
   newline();
   return FALSE;
}


void get_dir(char path[]){
#ifdef MSDOS
   if (getcwd(path,255) != 0) return;
   sprintf(path,mystrerror(errno)); return;
#endif
#ifdef UNIX
   getcwd(path,255);
#endif
#ifdef VMS
   getcwd(path,255,1);
#endif
}



static char save_filename[80];

char *last_filename(){ return (save_filename); }
/* Returns a pointer to the last non-empty filename read by get_filename */


void add_default_extension(char filnam[], char wildcard[], int wild_allowed){
/* adds the extension from "wildcard" to "filnam", only if "filnam" */
/* has no extension, e.g. "test" "*.par"  -----> "test.par"         */
   char *p2, *tmp;
   tmp = filnam;
   /* tmp = pointer to separator between path and name in filnam */
   if (strrchr(filnam,PATHSEP)) tmp = strrchr(filnam,PATHSEP);
   if (strrchr(tmp,'.')) return;  /* filnam has already extension */
   /* tmp = pointer to separator between path and name in wildcard */
   tmp = wildcard;
   if (strrchr(wildcard,PATHSEP)) tmp = strrchr(wildcard,PATHSEP);
   /* tmp = pointer to "." in filename part of wildcard */
   p2 = strrchr(tmp,'.');
   if (p2==NULL) return;  /* wildcard has no extension */
   if (! wild_allowed){
     /* Don't add extension if it contains wildcards */
     if (strchr(p2,'*') != NULL) return;
     if (strchr(p2,'?') != NULL) return;
     if (strchr(p2,'%') != NULL) return;
   }
   strcat(filnam,p2);
}

void get_filename(FILE *volterm,char filnam[], char frage[], char wildcard[]){
    int i,j1,j2,k;
    char c0,cp, ext[5], back[6];
    c0='\0';
    cp='.';
    back[0]='\x1B';
    back[1]='['; back[2]='2'; back[3]='A';
    back[4]=c0;
 show:
      fprintf(volterm,"  %s [%s]\n",frage,wildcard);
      i=0;while(wildcard[i]!=cp)i++;j1=i;
      i=0;while(wildcard[i]!=c0)i++;j2=i;

      /*fprintf(volterm,"j1=%d j2=%d \n",j1,j2);*/
      i=0; for(k=j1;k<j2;k++){ext[i]=wildcard[k];i++;}ext[i]=c0;

      fget_string(volterm,filnam);

      if (filnam[0]=='\0'){strcpy(filnam,wildcard);
      }else{
       i=0;while(filnam[i]!=c0)i++;j1=i;
       i=0;while(filnam[i]!=cp && i<j1)i++;j2=i;
       if(j1==j2)strcat(filnam,ext);
       strcpy(wildcard,filnam);
       fprintf(volterm,"%s",back);
       fprintf(volterm,"                                                 \n");
       fprintf(volterm,"                                                 \n");
       fprintf(volterm,"%s",back);
       fprintf(volterm,"%s",back);
       goto show;
      }
}

FILE *lesefile_oeffnen(FILE *volterm,char frage[], char wildcard[])
{ char name[80];
  FILE *infile;
  fprintf(volterm,"===if no file available exit with: e ===\n\n");
  do {
        get_filename(volterm,name,frage,wildcard);
        if (name[0]=='e') return(NULL);
	if (!(infile = fopen(name,"r"))) {
	  fprintf(volterm,"    Error opening %s for input\n",name);
          fprintf(volterm,  "    --- %s\n",mystrerror(errno));
	}
  } while (!infile);
  return(infile);
}

FILE *schreibfile_oeffnen(FILE *volterm,char frage[], char wildcard[])
{ char name[80];
  FILE *infile;
fprintf(volterm,"=== exit with: e layout not saved ===\n\n");
  do {
        get_filename(volterm,name,frage,wildcard);
        if (name[0]=='e') return(NULL);
	if (!(infile = fopen(name,"w"))) {
	  fprintf(volterm," Error opening %s for output\n",name);
          fprintf(volterm,"      --- %s ---\n",mystrerror(errno));
	}
  } while (!infile);
  return(infile);
}

int get_key()
{
#ifdef BORLANDC
   int c;
   c = getch();
   if (c == LF) return(CR);
   if (c) return(c);
   switch (getch()) {
      case  72 : return(SMG_K_TRM_UP);
      case  80 : return(SMG_K_TRM_DOWN);
      case  77 : return(SMG_K_TRM_RIGHT);
      case  75 : return(SMG_K_TRM_LEFT);
      default : return(0);
  }
#endif
#ifdef GNUPC
   int c;
   c = getkey();
   if (c == LF) return(CR);
   if (c<256) return(c);
   switch (c) {
      case  328 : return(SMG_K_TRM_UP);
      case  336 : return(SMG_K_TRM_DOWN);
      case  333 : return(SMG_K_TRM_RIGHT);
      case  331 : return(SMG_K_TRM_LEFT);
      default : return(0);
  }
#endif
#ifdef UNIX
   char c;
   refresh(); /* ncurses */
   c = getch();
   if (c == LF) return(CR);
   if (c != ESC) return(c);
   if ((getch()) != '[') return(0);
   switch (getch()) {
      case  'A' : return(SMG_K_TRM_UP);
      case  'B' : return(SMG_K_TRM_DOWN);
      case  'C' : return(SMG_K_TRM_RIGHT);
      case  'D' : return(SMG_K_TRM_LEFT);
      default : return(0);
  }
#endif
#ifdef VMS
    unsigned short  termcode;
    smg$read_keystroke(&keyb_id,&termcode);
    if (termcode == LF) return(CR);
    if (termcode < 256) return(termcode);
    switch (termcode) {
      case  SMG$K_TRM_UP    : return(SMG_K_TRM_UP);
      case  SMG$K_TRM_DOWN  : return(SMG_K_TRM_DOWN);
      case  SMG$K_TRM_RIGHT : return(SMG_K_TRM_RIGHT);
      case  SMG$K_TRM_LEFT  : return(SMG_K_TRM_LEFT);
      default : return(0);
    }
#endif
}
void fget_string(FILE *volterm,char strng[]) /* ncurses */
{
  int i;
  /*  refresh();*/
  /*getstr(strng);*/
  i=0;while((strng[i]=getc(volterm))!= '\n')i++;
  strng[i]='\0';
}
void get_string(char strng[]) /* ncurses */
{
  int i;
#ifdef UNIX
  /*  refresh();*/
  /*getstr(strng);*/
  i=0;while((strng[i]=getchar())!= '\n')++i;
  strng[i]='\0';
#else
  gets(strng);
#endif
}


int read_key(){
   int c;
#ifdef UNIX
   noecho();
   cbreak();
#endif
   c = get_key();
#ifdef UNIX
   nocbreak();
   echo();
#endif
   return(c);
}

void gotoxy(int x, int y)
{
   printf("%c[%1d;%1dH",ESC,y,x);
}

int KeyPressed(){
#ifdef VMS
    struct SMG$R_ATTRIBUTE_INFO_BLOCK p_kit;
    smg$get_keyboard_attributes(&keyb_id, &p_kit, &SMG$C_KEYBOARD_INFO_BLOCK);
    return(p_kit.SMG$R_COUNT_OVERLAY.SMG$W_TYPEAHD_CNT != 0);
#else
    return(0);
#endif
}

int smg_key_pressed(){
#ifdef VMS
    struct SMG$R_ATTRIBUTE_INFO_BLOCK p_kit;
    smg$get_keyboard_attributes(&keyb_id, &p_kit, &SMG$C_KEYBOARD_INFO_BLOCK);
    return(p_kit.SMG$R_COUNT_OVERLAY.SMG$W_TYPEAHD_CNT != 0);
#else
    return(0);
#endif
}


void lib_wait(float t){
/* #ifdef VMS
  lib$wait(&t);
  #else */
  int it;
  it = t;
  sleep(it);
/* #endif */
}

int entscheidung(FILE *volterm,char frage[], char def)
{
   char instr[80], ch;
   def = toupper(def);
   newline();
   fprintf(volterm,"%s  (D=%c) ? ",frage,def);
   do
     {
	fget_string(volterm,instr);
	if (! instr[0]) ch = def;
        else ch = toupper(instr[0]);
        if (ch == 'N') return(0);
        if ((ch == 'Y')||(ch == 'J')) return(1);
     }
   while(1);
}

int entscheidung_a(char frage[], int def, int anz)
{
   static char defchar[4] = "YNE";
   char ch,instr[80];
   if (anz == 2)  
     {
       newline();
       printf("%s [Yes/No] (D=%c) ? ",frage,defchar[def-1]);
     }
   else 
     {
       newline();
       printf("%s [Yes/No/Exit] (D=%c) ? ",frage,defchar[def-1]);
     };
   do
     {
	get_string(instr);
	if (! instr[0]) return(def);
        else ch = toupper(instr[0]);
        if (ch == 'N') return(2);
        if ((ch == 'Y')||(ch == 'J')) return(1);
        if ((ch == 'E') && (anz==3)) return(3);
     }
   while(1);
}


void vaxdate(char datum[] ){
  time_t t;
  char *ct;
  t = time(NULL);
  ct = ctime(&t);
  strncpy(datum,ct,10);
  datum[10] = 0;
  }

void vaxtime(char zeit[] ){
  time_t t;
  char *ct;
  t = time(NULL);
  ct = ctime(&t);
  strncpy(zeit,ct+11,10);
}




/************************************************************************/
/* trans : get float values from a string                               */
/*         returns : number of values found                             */
/*         instr   : string to be evaluated                             */
/*         out     : array with values                                  */
/************************************************************************/

/* Return nearest integer to x */
/*int round(double x){
  int i;
  i = (int)(floor(x+0.5));
  if ((double)i < x-0.51) i++; 
  if ((double)i > x+0.51) i--;
  return i;
  } */

int trans(char instr[], double out[]){

/* This procedure looks how many numerical values are in "instr" */
/* It returns the number of values and the values in "out".      
  char instr[];
  double out[];
    printf("trans  instr=%s \n",instr);*/
    char *ich = ".-+Ee 0123456789";
    char *tmp;
    int nnum,nex,x,k,f,l,iout;
    double pt, nsig;
    tmp = malloc(strlen(instr)*sizeof(char)+2);
    strcpy(tmp,instr); 
    strcat(tmp," "); /* terminating blank */
    nsig = plus; pt = 1.0; nex = nnum = iout=0;
    for (k = 0; k < 16; out[k++] = 0.0);   /* initialize out       */
    for (k = 0; (tmp[k] != NUL) ; k++)   /* scan instr until NUL */
    { if (iout > 15) break;                /* outarray full        */
      x = (int)strchr(ich,tmp[k]) - (int)ich + 1;
      if (x >= 7)
      { nnum=1;
        if (pt > 0.9) out[iout] = out[iout]*10.0 + (x-7);
        else
        { out[iout] += ((x-7)*pt);
          pt /= 10;
      }}
      if (x == 1) pt=0.1;
      if (x == 2) nsig=minus;
      if ((x != 1) && (x < 7))
      { if (nnum)
          { out[iout] *= nsig;
            if (out[iout] >= 1.0) out[iout] += 1.0E-9;
            nsig=plus; nnum=0; pt=1.0;
            if (nex)
            { if (iout)
              { f = round(out[iout]);
                if (f > 0) for (l=1; l <= f; l++)
                  out[iout-1] *= 10.0;
                else for (l= -1; l >= f; l--)
                  out[iout-1] /=  10.0;
              }
              nex=0; out[iout]=0; iout=iout-1;
            }
            iout++;
          }
          if (((x == 4) || (x == 5)) && (iout > 0) &&
	      (strchr(" .0123456789",tmp[k-1]))) nex=1;
  }}
  free(tmp);
  return iout;
}


#ifdef VMS

#include <sjcdef.h>
#include <jbcmsgdef.h>
#include <ssdef.h>

typedef struct itemlist{
   short int buflen, itemcode;
   void *buffer;
   int *retadr;
} itemlist;

typedef struct iobuffer {
   unsigned long int io_stat;
   long int dev_info;
} iobuffer;
   
int sys$sndjbcw(unsigned efn,short int func,unsigned null,
   itemlist *items,iobuffer *iosb, unsigned *astadr, unsigned *astprm);
          

static int start_job(char file[], char queue[], int delete, 
                     int *entry, char statstr[]){
   int stat, len, ok, dummy;
   struct itemlist item[7];
   struct iobuffer iosb;
   item[0].itemcode = SJC$_QUEUE;   item[0].buflen = strlen(queue);
   item[0].buffer = queue;    item[0].retadr = NULL;
   item[1].itemcode = SJC$_FILE_SPECIFICATION; item[1].buflen = strlen(file);
   item[1].buffer = file;    item[1].retadr = NULL;
   item[2].itemcode = SJC$_ENTRY_NUMBER_OUTPUT; item[2].buflen = 4;
   item[2].buffer = entry;   item[2].retadr = &dummy;
   item[3].itemcode = SJC$_JOB_STATUS_OUTPUT; item[3].buflen = 255;
   item[3].buffer = statstr;    item[3].retadr = &len;
   if (delete) item[4].itemcode = SJC$_DELETE_FILE;    
          else item[4].itemcode = SJC$_NO_DELETE_FILE;    
   item[4].buflen = 0;  
   item[4].buffer = NULL;       item[4].retadr = NULL;
   item[5].itemcode = SJC$_NO_LOG_SPOOL;      item[5].buflen = 0;  
   item[5].buffer = NULL;       item[5].retadr = NULL;
   item[6].itemcode = 0;                      item[6].buflen = 0;  
   item[6].buffer = NULL;       item[6].retadr = NULL;
   stat = sys$sndjbcw(0,SJC$_ENTER_FILE,0,item,&iosb,NULL,NULL);
   ok = ((stat == SS$_NORMAL) || (stat == JBC$_NORMAL));
   ok = (ok  && ((iosb.io_stat == SS$_NORMAL) 
              || (iosb.io_stat == JBC$_NORMAL)
#ifdef ALPHA
              || (iosb.io_stat == JBC$_ITMREMOVED)
#endif
   ));
   if (ok) statstr[len] = 0;
   else strcpy(statstr,"***ERROR***");
   if (! ok) lib$signal(iosb.io_stat | 1);
   if (! ok) lib$signal(stat | 1);
   return ok;   
}
#endif

void VMS_submit(char outputdevice[], char filename[], int delete){
#ifdef VMS
   char statstr[255];
   int entry;  
   if (start_job(filename, outputdevice, delete, &entry, statstr))
        printf(statstr); 
   else 
     {
       printf(" Error starting job %s on queue %s",filename,outputdevice);
       newline();
     };
#elif defined(UNIX)
   char outstr[255];
   sprintf(outstr,"lpr -P%s %s",outputdevice, filename);
   system(outstr);
#endif
}
