      subroutine lineq(ncase,nd,u,dp)
      implicit none
      integer i,j,l,k,kk,ncase,nd,k1,ll,ii,jj,j1
      real sum,u,dp
      dimension u(nd,nd),dp(nd)

c     ******************************************************************
c     solving systems of linear equations by the method of cholesky
c     calculation of inverse matrix
c     ******************************************************************
c     ncase = 0   linear equations, u = left sides, dp = right sides
c     solution stored in dp
c     ncase = 1   calculation of the inverse matrix of u
c     stored in u
c     u = symmetrical nd*nd-matrix, only upper triangle is required
c     dp = vector(nd), dummy for matrix inversion
c     ******************************************************************
c
c     choleski
c
      do k=1,nd
       kk=k-1
       if (kk.gt.0) then
        do k1=1,kk
         if (ncase.eq.0) dp(k)=dp(k)-u(k1,k)*dp(k1)
         do l=k,nd
          u(k,l)=u(k,l)-u(k1,l)*u(k1,k)
         end do
        end do
       end if

       if (u(k,k).gt.0) then
        u(k,k)=sqrt(u(k,k))
       else if(u(k,k).lt.0) then
        u(k,k)=sqrt(-u(k,k))
       end if
       if (u(k,k).lt.1.e-4) u(k,k)=1.e-4

       if (ncase.eq.0) dp(k)=dp(k)/u(k,k)
 
       kk=k+1
       if (nd-kk.lt.0) goto 60
       do l=kk,nd
        u(k,l)=u(k,l)/u(k,k)
       end do
      end do
c
c     back substitution
c
   60 if (ncase.eq.1) goto 100

      do k=1,nd
       kk=nd-k+1
       if (k-1.ne.0)then 
        ll=kk+1
        do l=ll,nd
         dp(kk)=dp(kk)-u(kk,l)*dp(l)
        end do
       end if
       dp(kk)=dp(kk)/u(kk,kk)
      end do

      goto 160
c
c     calculation of the inverse matrix
c
  100 do kk=1,nd
       k=nd-kk+1
       do ii=kk,nd
        i=nd-ii+1
        jj=i+1
        sum =0.
        if(i-nd.eq.0) goto 115
        do j=jj,nd
         j1=j
         k1=k
         if(k-j.lt.0) then
          k1=j
          j1=k
         end if
         sum=sum-u(i,j)*u(i,i)*u(k1,j1)
        end do
  115   if (k.eq.i) sum=sum+1

        u(k,i)=sum/u(i,i)**2
       end do
      end do

      do  k=1,nd
       do  j=k+1,nd
        u(k,j)=u(j,k)
       end do
      end do

 160  return
c     write(6,'(4g15.4)')(dp(k),k=1,nd)
      end

      function FUNC(pp,mf)
      implicit none
      include 'comglb.for'
      include 'compar.for'
      include 'comfrm.for'
      include 'comdat.for'
      include 'comdas.for'
      include 'comthe.for'
      include 'comths.for'
      integer mf
      real FUNC,pp,z

      integer   ib_df,kana,ibmisfit
      real      thfk,dfdp,ddpk
      common/fr2fi/thfk(max8),dfdp(max8),ddpk,ib_df,kana,ibmisfit

      dimension pp(mf)
      integer i,kpa,info,j,j1,k,ks,nf,nchansp

      do i=1,mf
       kpa=kf(i)
       if(i_opgtup(i).eq.1) then
        if(pp(i).gt.op(kpa))then
         pp(i)=op(kpa)
        else if(pp(i).lt.up(kpa))then
         pp(i)=up(kpa)
        end if
       end if
       p(kpa)=pp(i)
      end do
      call calpl('free')


c     parameter p defined
c      write(6,'(a,10g12.3)')'FUNC pp(..)=',(pp(i),i=1,mf)
       info=0
       z=0.
       nf=0
       do k=1,nfit(0)
        ks=nfit(k)
        if(b_datset(ks))then
         j1=1
         call frametheory(ks,'test',j1,info)
         if(info.gt.0) goto 10
         nchansp=cdata(ks,ip_chan(ks))
c        call arrtoks(ks)
         do j=2,ichanks(0),2
          do i=ichanks(j-1),ichanks(j)
           nf=nf+1
           z=z+((real_fr(mfpt+i)-thfk(i))/real_fr(mdfpt+i))**2
          end do
         end do
        end if
       end do
       if(nf.gt.0) then
        FUNC=z/(nf-mf)
c       write(6,'(a,2g15.5)') ' powell func=',FUNC
        return
       end if
 10    FUNC=0.
c       write(6,'(a,i4)') ' powell from frametheory info=',info
       return
       end

      subroutine DFUNC(pp,dpp,f0,df,mf)
      implicit none
      integer mf,i
      real fd,f0,FUNC,pp,dpp,df
      dimension pp(mf),dpp(mf),df(mf)

      
       f0=FUNC(pp,mf)
       do i=1,mf
        pp(i)=pp(i)+dpp(i)
        fd=FUNC(pp,mf)
        pp(i)=pp(i)-dpp(i)
        df(i)=(fd-f0)/dpp(i)
       end do
      return
      end

      subroutine DDFUNC(pp,dpp,ddf,mf)
      implicit none

      integer mf,i,k
      real FUNC,pp,dpp,ddf,f0,fdi,fdk,fdii,fdik
      dimension pp(mf),dpp(mf),ddf(mf,mf)


      f0=FUNC(pp,mf)
c      write(6,'(a,g15.5,i4)')' DDFUNC f0,nf=',f0,nf
      do i=1,mf
       pp(i)=pp(i)+dpp(i)
       fdi=FUNC(pp,mf)
       pp(i)=pp(i)+dpp(i)
       fdii=FUNC(pp,mf)
       pp(i)=pp(i)-2.0*dpp(i)
       ddf(i,i)=(fdii-2.0*fdi+f0)/dpp(i)**2
c      write(6,'(2i4,a,6g12.4)')i,i,' DDFUNC ddf=',ddf(i,i)
       do k=i+1,mf
        pp(k)=pp(k)+dpp(k)
        fdk=FUNC(pp,mf)
        pp(i)=pp(i)+dpp(i)
        fdik=FUNC(pp,mf)
        pp(k)=pp(k)-dpp(k)
        pp(i)=pp(i)-dpp(i)
        ddf(i,k)=(fdik-fdi-fdk+f0)/dpp(i)/dpp(k)
        ddf(k,i)=ddf(i,k)
c       write(6,'(2i4,a,6g12.4)')i,k,' DDFUNC ddf=',ddf(i,k)
       end do
      end do
      return
      end


      SUBROUTINE POWELL(PP,dpp,XX,N,ITER,
     &                  FTOL,ITERBRAK,iBR_DBR,ibx_calc,
     &                  df,PT,PTT,XIT,ITER_M,
     &                  kf,namepp,pplast,nwr,FRET)
      implicit none

      character namepp*24
      real PP,dpp,XX,df,PT,PTT,XIT,pplast,FTOL,FRET
      Integer N,ITER,ITERBRAK,ITER_M,iBR_DBR,ibx_calc,kf,nwr
      DIMENSION PP(N),dpp(N),XX(N,N),df(N),PT(N),PTT(N),
     &          XIT(N),pplast(N),namepp(N)
      dimension ITER_M(N),kf(N)
      integer I,J,ITMAX,IBIG,ITERM
      real FUNC,DEL,FP,FPTT,T

      do I=1,N
      do J=1,N
       XX(I,J)=0.0
      end do
      end do
      do I=1,N
       XX(I,I)=dpp(I)
      end do

      FRET=FUNC(PP,N)
      DO 11 J=1,N
        PT(J)=PP(J)
 11   CONTINUE
c     input iter is maximal iterations; output actual iterations
      ITMAX=ITER
      ITER=0
 1    ITER=ITER+1
      FP=FRET
      IBIG=0
      DEL=0.
      ITERM=0
      write(nwr,'(i4,a)')ITER,'. iteration: Powell'
        write(nwr,'(2x,a2,5x,a4,24x,a,12x,a,10x,a)')
     &                'nu','name','value','d-value','int. loops'

      DO 13 I=1,N
        DO 12 J=1,N
          XIT(J)=XX(J,I)
 12     CONTINUE
c        write(6,'(a,i4)')' powell i=',I
        CALL LINMIN(PP,dpp,PTT,df,XIT,N,FRET,
     &              FTOL,ITERBRAK,iBR_DBR,ibx_calc,ITERM)
c        CALL LINMIN(PP,PTT,XIT,N,FRET,FTOL,ITERBRAK,ITERM)
        ITER_M(I)=ITERM
        IF(ABS(FP-FRET).GT.DEL)THEN
          DEL=ABS(FP-FRET)
          IBIG=I
        ENDIF
 13   CONTINUE

c      do i=1,N
c       write(nwr,'(8g15.5,)')(XX(j,i),j=1,N)
c      end do

      do I=1,N
       write(nwr,'(x,a1,i3,a1,2x,a24,g15.5,2x,g15.5,8x,i3)')
     &    '(',kf(I),')',namepp(I),PP(I),PP(I)-pplast(I), ITER_M(I)
       end do
       write(nwr,'(a,g15.5)') ' chi**2=',FRET


      IF(2.0*ABS(FP-FRET).LE.FTOL*(ABS(FP)+ABS(FRET)))then
       write(nwr,'(a,f12.5,a)')
     &  'local minimum  (|d chi**2|/chi**2 <',FTOL,') reached'
       RETURN
      end if
      IF(ITER.EQ.ITMAX) then
       write(nwr,'(a)')' termination without convergence'
       RETURN
      end if
      DO J=1,N
        PTT(J)=2.0*PP(J)-PT(J)
        XIT(J)=PP(J)-PT(J)
        PT(J)=PP(J)
      end do
      FPTT=FUNC(PTT,N)

      IF(FPTT.GE.FP) GOTO 1
      T=2.0*(FP-2.0*FRET+FPTT)*(FP-FRET-DEL)**2-DEL*(FP-FPTT)**2
      IF(T.GE.0.0) GOTO 1

       CALL LINMIN(PP,dpp,PTT,df,XIT,N,FRET,
     &             FTOL,ITERBRAK,iBR_DBR,ibx_calc,ITERM)
      write(nwr,'(a,g15.5)') ' chi**2=',FRET

      DO J=1,N
        XX(J,IBIG)=XIT(J)
      end do

      GO TO 1
      END

      SUBROUTINE LINMIN(PP,dpp,PTT,df,XI,N,FRET,
     &                  FTOL,iBR_DBR,ibx_calc,ITERBRAK,ITERM)
      implicit none
      integer N,ITERBRAK,iBR_DBR,ibx_calc,ITERM
      real PP,dpp,PTT,df,XI,FTOL,FRET
      DIMENSION PP(N),dpp(N),PTT(N),df(N),XI(N)
      real ax,bx,cx,XMIN,DBRENT,BRENT
      integer ITER,J
      ax=0.0
      bx=1.0
      if(ibx_calc.eq.1) then
       do J=1,N
        if(abs(bx*XI(J)).gt.dpp(J)) bx=abs(dpp(J)/XI(J))
       end do
      end if
      cx=2.0*bx
      ITER=ITERBRAK
      CALL MNBRAK(PP,PTT,XI,N,ax,bx,cx,ITER)
      if(iBR_DBR.eq.1) then
       FRET=BRENT(PP,PTT,XI,N,ax,bx,cx,FTOL,XMIN,ITER)
      else
       FRET=DBRENT(PP,dpp,PTT,df,XI,N,ax,bx,cx,FTOL,XMIN,ITER)
      end if
      if(ITERM.lt.ITER)ITERM=ITER
      DO 12 J=1,N
        XI(J)=XMIN*XI(J)
        PP(J)=PP(J)+XI(J)
 12   CONTINUE
      RETURN
      END

      SUBROUTINE MNBRAK(PP,PTT,XI,N,AX,BX,CX,ITER)
      implicit none
      real PP,PTT,XI,AX,BX,CX,GOLD,GLIMIT,TINY
      integer N,ITER,ITMAX
      dimension PP(N),PTT(N),XI(N)
      PARAMETER (GOLD=1.618034, GLIMIT=10.0, TINY=1.0e-6)
      REAL U,FA,FB,FC,FU,F1DIM,DUM,R,Q,ULIM


c      write(6,'(a,6g15.6)')' mnbrak XI:',XI
c      write(6,'(a,6g15.6)')' mnbrak PP:',PP
      FA=F1DIM(PP,PTT,XI,N,AX)
c      write(6,'(a,6g15.6)')' mnbrak AX,PTT:',AX,PTT
      FB=F1DIM(PP,PTT,XI,N,BX)
c      write(6,'(a,6g15.6)')' mnbrak BX,PTT:',BX,PTT
      
      IF(FB.GT.FA)THEN
        DUM=AX
        AX=BX
        BX=DUM
        DUM=FB
        FB=FA
        FA=DUM
      ENDIF
      CX=BX+GOLD*(BX-AX)
      FC=F1DIM(PP,PTT,XI,N,CX)
c      write(6,'(a,6g15.6)')' mnbrak :',AX,BX,CX,FA,FB,FC
      ITMAX=ITER
      ITER=-1
 1    IF(FB.GE.FC)THEN
        ITER=ITER+1
c        write(6,'(a,2i4)')'MNBRAK ITMAX,iter=',ITMAX,ITER
        if(ITER.gt.ITMAX) goto 2
        R=(BX-AX)*(FB-FC)
        Q=(BX-CX)*(FB-FA)
        U=BX-((BX-CX)*Q-(BX-AX)*R)/
     &                (2.0*SIGN(MAX(ABS(Q-R),TINY),Q-R))
        ULIM=BX+GLIMIT*(CX-BX)
        IF((BX-U)*(U-CX).GT.0.0)THEN
          FU=F1DIM(PP,PTT,XI,N,U)
c          write(6,'(a,5g15.5)')'1MNBRAK U=',U,FU
          IF(FU.LT.FC)THEN
            AX=BX
            FA=FB
            BX=U
            FB=FU
            GO TO 1
          ELSE IF(FU.GT.FB)THEN
            CX=U
            FC=FU
            GO TO 1
          ENDIF
          U=CX+GOLD*(CX-BX)
          FU=F1DIM(PP,PTT,XI,N,U)
c          write(6,'(a,5g15.5)')'2MNBRAK U=',U,FU
        ELSE IF((CX-U)*(U-ULIM).GT.0.0)THEN
          FU=F1DIM(PP,PTT,XI,N,U)
c          write(6,'(a,5g15.5)')'3MNBRAK U=',U,FU
          IF(FU.LT.FC)THEN
            BX=CX
            CX=U
            U=CX+GOLD*(CX-BX)
            FB=FC
            FC=FU
            FU=F1DIM(PP,PTT,XI,N,U)
          ENDIF
        ELSE IF((U-ULIM)*(ULIM-CX).GE.0.0)THEN
          U=ULIM
          FU=F1DIM(PP,PTT,XI,N,U)
c          write(6,'(a,5g15.5)')'4MNBRAK U=',U,FU
        ELSE
          U=CX+GOLD*(CX-BX)
          FU=F1DIM(PP,PTT,XI,N,U)
c          write(6,'(a,5g15.5)')'5MNBRAK U=',U,FU
        ENDIF
        AX=BX
        BX=CX
        CX=U
        FA=FB
        FB=FC
        FC=FU
        if(abs(FA-FB)+abs(FB-FC).lt.TINY) then
         ITER=1
         goto 2
        end if
        GO TO 1
      ENDIF
      ITER=ITMAX
 2    RETURN
      END

      FUNCTION BRENT(PP,PTT,XI,N,AX,BX,CX,TOL,XMIN,ITER)
      implicit none

      real BRENT,PP,PTT,XI,AX,BX,CX,TOL,XMIN,CGOLD,TINY
      integer N,ITER,ITMAX
      dimension PP(N),PTT(N),XI(N)
      REAL U,FU,F1DIM,R,Q,TOL1,TOL2,
     &     A,B,V,W,X,E,FX,FV,FW,XM,P,ETEMP,D

      PARAMETER (CGOLD=.381966,TINY=1.0e-6)

      A=MIN(AX,CX)
      B=MAX(AX,CX)
      V=BX
      W=V
      X=V
      E=0.0
      FX=F1DIM(PP,PTT,XI,N,X)
      FV=FX
      FW=FX
      ITMAX=ITER
c      write(6,'(a,6g15.6)')' mnbrak :',BX,FX
      DO 11 ITER=1,ITMAX
c        write(6,'(a,2i4)')'BRENT ITMAX,iter=',ITMAX,ITER

        XM=0.5*(A+B)
        TOL1=TOL*ABS(X)+TINY
        TOL2=2.0*TOL1
        IF(ABS(X-XM).LE.(TOL2-0.5*(B-A))) GOTO 3
        IF(ABS(E).GT.TOL1) THEN
          R=(X-W)*(FX-FV)
          Q=(X-V)*(FX-FW)
          P=(X-V)*Q-(X-W)*R
          Q=2.0*(Q-R)
          IF(Q.GT.0.0) P=-P
          Q=ABS(Q)
          ETEMP=E
          E=D
          IF(ABS(P).GE.ABS(0.5*Q*ETEMP).OR.P.LE.Q*(A-X).OR.
     *        P.GE.Q*(B-X)) GOTO 1
          D=P/Q
          U=X+D
          IF(U-A.LT.TOL2 .OR. B-U.LT.TOL2) D=SIGN(TOL1,XM-X)
          GOTO 2
        ENDIF
1       IF(X.GE.XM) THEN
          E=A-X
        ELSE
          E=B-X
        ENDIF
        D=CGOLD*E
 2      IF(ABS(D).GE.TOL1) THEN
          U=X+D
        ELSE
          U=X+SIGN(TOL1,D)
        ENDIF
        FU=F1DIM(PP,PTT,XI,N,U)
c        write(6,'(a,2F10.3)')'BRENT FU=',U,FU
        IF(FU.LE.FX) THEN
          IF(U.GE.X) THEN
            A=X
          ELSE
            B=X
          ENDIF
          V=W
          FV=FW
          W=X
          FW=FX
          X=U
          FX=FU
        ELSE
          IF(U.LT.X) THEN
            A=U
          ELSE
            B=U
          ENDIF
          IF(FU.LE.FW .OR. W.EQ.X) THEN
            V=W
            FV=FW
            W=U
            FW=FU
          ELSE IF(FU.LE.FV .OR. V.EQ.X .OR. V.EQ.W) THEN
            V=U
            FV=FU
          ENDIF
        ENDIF
 11   CONTINUE
c      write(6,'(a)') 'Brent exceed maximum iterations.'
 3    XMIN=X
      BRENT=FX
      RETURN
      END


      FUNCTION F1DIM(PP,PTT,XI,N,X)
      implicit none
      integer N,J
      real F1DIM,PP,PTT,XI,X,FUNC
      DIMENSION PP(N),PTT(N),XI(N)

      DO 11 J=1,N
        PTT(J)=PP(J)+X*XI(J)
 11   CONTINUE
      F1DIM=FUNC(PTT,N)
c      write(6,'(a,i4,10g12.5)')' F1DIM N,X,XT(J)=',
c     &                        N,F1DIM,X,(PTT(J),J=1,N)
      RETURN
      END

      SUBROUTINE DFPMIN(PP,dpp,HESSIN,N,ITER,
     &                  FTOL,ITERBRAK,iBR_DBR,ibx_calc,
     &                  df,DG,HDG,XIT,ITER_M,
     &                  kf,namepp,pplast,nwr,FRET)
      implicit none

      character namepp*24
      real PP,dpp,HESSIN,df,DG,HDG,XIT,pplast,FTOL,FRET
      Integer N,ITER,ITERBRAK,ITER_M,iBR_DBR,ibx_calc,kf,nwr
      DIMENSION PP(N),dpp(N),HESSIN(N,N),df(N),DG(N),HDG(N),
     &          XIT(N),pplast(N),namepp(N)
      dimension ITER_M(N),kf(N)

      integer I,J,ITS,ITMAX,ITERM
      real FP,TINY,FAD,FAE,FAC

      PARAMETER (TINY=1.E-6)

      do I=1,N
      do J=1,N
       HESSIN(I,J)=0.0
      end do
      end do
      do I=1,N
       HESSIN(I,I)=1.0
      end do


      call DFUNC(PP,dpp,FP,df,N)
      DO I=1,N
        XIT(I)=-df(I)
         DG(I)=df(I)
      end do
      ITMAX=ITER
      ITER=0
      DO 24 ITS=1,ITMAX
      ITERM=0

      ITER=ITS
      write(nwr,'(i4,a)')ITER,'. iteration: D.F.P.'
        write(nwr,'(2x,a2,5x,a4,24x,a,12x,a)')
     &                'nu','name','value','d-value'

        CALL LINMIN(PP,dpp,HDG,df,XIT,N,FRET,
     &               FTOL,ITERBRAK,iBR_DBR,ibx_calc,ITERM)

        do I=1,N
         write(nwr,'(x,a1,i3,a1,2x,a24,g15.5,2x,g15.5)')
     &    '(',kf(I),')',namepp(I),PP(I),PP(I)-pplast(I)
        end do
        write(nwr,'(a,g15.5,a,i3)') ' chi**2=',FRET,
     &                              ' internal loops=',ITERM

        IF(2.*ABS(FRET-FP).LE.FTOL*(ABS(FRET)+ABS(FP)+TINY)) then
         write(nwr,'(a,f12.5,a)')
     &  'local minimum  (|d chi**2|/chi**2 <',FTOL,') reached'
         RETURN
        end if
        IF(ITER.EQ.ITMAX) then
         write(nwr,'(a)')' termination without convergence'
         RETURN
        end if
        FP=FRET
        CALL DFUNC(PP,dpp,FRET,df,N)
        DO I=1,N
          DG(I)=df(I)-DG(I)
        end do
c        write(6,'(a,6g12.3)')' DFPMIN DG=',DG        
        DO I=1,N
          HDG(I)=0.
          DO J=1,N
            HDG(I)=HDG(I)+HESSIN(I,J)*DG(J)
          end do
        end do
c        write(6,'(a,6g12.3)')' DFPMIN HESSIN=',HESSIN        
c        write(6,'(a,6g12.3)')' DFPMIN HDG   =',HDG        

        FAC=0.
        FAE=0.
        DO I=1,N
          FAC=FAC+DG(I)*XIT(I)
          FAE=FAE+DG(I)*HDG(I)
        end do
        FAC=1./FAC
        FAD=1./FAE
        DO I=1,N
          DG(I)=FAC*XIT(I)-FAD*HDG(I)
        end do
c        write(6,'(a,6g12.3)')' DFPMIN new DG=',DG        

        DO I=1,N
          DO J=1,N
            HESSIN(I,J)=HESSIN(I,J)+FAC*XIT(I)*XIT(J)
     *        -FAD*HDG(I)*HDG(J)+FAE*DG(I)*DG(J)
          end do
        end do

        DO I=1,N
          XIT(I)=0.
          DO J=1,N
            XIT(I)=XIT(I)-HESSIN(I,J)*df(J)
          end do
        end do
c        write(6,'(a,6g12.3)')' DFPMIN newHESSIN=',HESSIN        
c        write(6,'(a,6g12.3)')' DFPMIN newXIT   =',XIT        
 24   CONTINUE
 100  continue
c       do I=1,N
c        write(6,'(8g12.5)')(HESSIN(i,k),k=1,N)
c       end do
      RETURN
      END

      FUNCTION DF1DIM(PP,dpp,PTT,df,XI,N,X)
      implicit none
      integer N,J
      real DF1DIM,PP,dpp,PTT,df,XI,X,F0
      DIMENSION PP(N),dpp(N),PTT(N),XI(N),df(N)

      DO 11 J=1,N
        PTT(J)=PP(J)+X*XI(J)
 11   CONTINUE
      CALL DFUNC(PTT,dpp,F0,df,N)
      DF1DIM=0.
      DO 12 J=1,N
        DF1DIM=DF1DIM+df(J)*XI(J)
 12   CONTINUE
      RETURN
      END



      FUNCTION DBRENT(PP,dpp,PTT,df,XI,N,AX,BX,CX,TOL,XMIN,ITER)
      implicit none

      real DBRENT,PP,dpp,PTT,df,XI,AX,BX,CX,TOL,XMIN,TINY
      integer N,ITER,ITMAX
      dimension PP(N),dpp(N),PTT(N),df(N),XI(N)
      REAL U,FU,F1DIM,DF1DIM,TOL1,TOL2,OLDE,
     &     A,B,V,W,X,E,FX,FV,FW,XM,D,DX,DU,DV,DW,D1,D2,U1,U2
      PARAMETER (TINY=1.0E-6)
      LOGICAL OK1,OK2




      A=MIN(AX,CX)
      B=MAX(AX,CX)
      V=BX
      W=V
      X=V
      E=0.
      FX=F1DIM(PP,PTT,XI,N,X)
c      write(6,'(a,6g15.6)')' DBRENT    PP:',PP
c      write(6,'(a,6g15.6)')' DBRENT  X,XI:',X,XI
c      write(6,'(a,6g15.6)')' DBRENT   PTT:',PTT
      FV=FX
      FW=FX
      DX=DF1DIM(PP,dpp,PTT,df,XI,N,X)
      DV=DX
      DW=DX
      ITMAX=ITER
c      write(6,'(a,6g15.6)')' DBRENT    df:',df

      DO 11 ITER=1,ITMAX
        XM=0.5*(A+B)
        TOL1=TOL*ABS(X)+TINY
        TOL2=2.*TOL1
c        write(6,'(a,4g12.6)')'DBRENT A,B,TOL1=',A,B,TOL*ABS(X)
        IF(ABS(X-XM).LE.(TOL2-.5*(B-A))) GOTO 3
        IF(ABS(E).GT.TOL1) THEN
          D1=2.*(B-A)
          D2=D1
          IF(DW.NE.DX) D1=(W-X)*DX/(DX-DW)
          IF(DV.NE.DX) D2=(V-X)*DX/(DX-DV)
          U1=X+D1
          U2=X+D2
          OK1=((A-U1)*(U1-B).GT.0.).AND.(DX*D1.LE.0.)
          OK2=((A-U2)*(U2-B).GT.0.).AND.(DX*D2.LE.0.)
          OLDE=E
          E=D
          IF(.NOT.(OK1.OR.OK2))THEN
            GO TO 1
          ELSE IF (OK1.AND.OK2)THEN
            IF(ABS(D1).LT.ABS(D2))THEN
              D=D1
            ELSE
              D=D2
            ENDIF
          ELSE IF (OK1)THEN
            D=D1
          ELSE
            D=D2
          ENDIF
          IF(ABS(D).GT.ABS(0.5*OLDE))GO TO 1
          U=X+D
          IF(U-A.LT.TOL2 .OR. B-U.LT.TOL2) D=SIGN(TOL1,XM-X)
          GOTO 2
        ENDIF
 1      IF(DX.GE.0.) THEN
          E=A-X
        ELSE
          E=B-X
        ENDIF
        D=0.5*E
 2      IF(ABS(D).GE.TOL1) THEN
          U=X+D
          FU=F1DIM(PP,PTT,XI,N,U)
        ELSE
          U=X+SIGN(TOL1,D)
          FU=F1DIM(PP,PTT,XI,N,U)
          IF(FU.GT.FX)GO TO 3
        ENDIF
        DU=DF1DIM(PP,dpp,PTT,df,XI,N,U)
        IF(FU.LE.FX) THEN
          IF(U.GE.X) THEN
            A=X
          ELSE
            B=X
          ENDIF
          V=W
          FV=FW
          DV=DW
          W=X
          FW=FX
          DW=DX
          X=U
          FX=FU
          DX=DU
        ELSE
          IF(U.LT.X) THEN
            A=U
          ELSE
            B=U
          ENDIF
          IF(FU.LE.FW .OR. W.EQ.X) THEN
            V=W
            FV=FW
            DV=DW
            W=U
            FW=FU
            DW=DU
          ELSE IF(FU.LE.FV .OR. V.EQ.X .OR. V.EQ.W) THEN
            V=U
            FV=FU
            DV=DU
          ENDIF
        ENDIF
 11   CONTINUE
c      PAUSE 'DBRENT exceeded maximum iterations.'
 3    XMIN=X
      DBRENT=FX
c      write(6,'(a,6g15.6)')' DBRENT  XMIN,FX:',XMIN,FX
      RETURN
      END





C====================================================================== 
C       PROGRAM: ran3.f
C       TYPE   : function
C       PURPOSE: generate random numbers
C       I/O    :
C	
C       VERSION: 27 MAI 94
C       COMMENT: Initialize idum with negative integer
Cnoprint===============================================================
        real function ran3(idum)
        Parameter (mbig=1000000000,mseed=161803398,mz=0,fac=1./mbig)
        dimension ma(55)
        save
        if (idum.lt.0.or.iff.eq.0) then
          iff=1
          mj=mseed-iabs(idum)
          mj=mod(mj,mbig)
          ma(55)=mj
          mk=1
          do 11 i=1,54
          ii=mod(21*i,55)
          ma(ii)=mk
          mk=mj-mk
          if (mk.lt.mz) mk=mk+mbig
          mj=ma(ii)
 11       continue
          do 13 k=1,4
            do 12 i=1,55
              ma(i)=ma(i)-ma(1+mod(i+30,55))
              if (ma(i).lt.mz)ma(i)=ma(i)+mbig
 12         continue
 13       continue
          inext=0
          inextp=31
          idum=1
        end if
        inext=inext+1
        if (inext.eq.56) inext=1
         inextp=inextp+1
        if (inextp.eq.56) inextp=1
        mj=ma(inext)-ma(inextp)
        if (mj.lt.mz) mj=mj+mbig
        ma(inext)=mj
        ran3=mj*fac
        end	

      subroutine mcarlodata(lq,chft,mf,pathw,iprobe,info)
      include 'comglb.for'
      include 'compar.for'
      include 'comfrm.for'
      include 'comdat.for'
      include 'comdas.for'
      include 'comthe.for'
      include 'comscd.for'
      include 'comths.for'


      integer   ib_df,kana,ibmisfit
      real      thfk,dfdp,ddpk
      common/fr2fi/thfk(max8),dfdp(max8),ddpk,ib_df,kana,ibmisfit

      integer   mff,ifree
      character ch2*2,formpr*14,ch8*8
      dimension  ifree(maxpf),kpfr(maxpf),chi_ar(maxpf),formpr(3)

      dimension length(0:4)
      character chft*4,name_theo*132,property*132
      logical   mosstrans,mossreem

      if(iprobe.gt.16)iprobe=16
      if(chft.eq.' ') then
       chft='newt'
c       write(6,'(a)')'no choice of fit routine ([ne] inserted)'
      end if
      i0=ichar('0')
      mf=0
      do kpa=1,mp
       if (kp(kpa).eq.1) then
        mf=mf+1
        ifree(mf)=kpa
        kf(mf)=kpa
        kpfr(mf)=kp(kpa)
       end if
      end do
      
      if(lq.lt.3)then
       if(mf.eq.0)then
        info=1
        infotext(1)='effi/command/mc/fit'
        infotext(2)='**error** at least one free parameter for the fit'
        infotext(3)=' '       
        return           
       end if
       npanum(maxc+2)=mf
       do j=1,mf
        npanum(j+maxc+2)=ifree(j)
       end do
        
       do k=0,4
        call fvadjust(k,length(k))
       end do
       i10=length(0)/10
       i1=length(0)-i10*10+i0
       ch2=char(i10+i0)//char(i1)
        
       formpr(1)='2a1,i1,3x,a'//ch2//','
       formpr(2)='2a1,i2,2x,a'//ch2//','
       formpr(3)='2a1,i3,1x,a'//ch2//','
        
c       write(6,'(a,5i4)')'fitsub info,iprobe,iterations,nfit(0)=',
c    &  mf,info,iprobe,iterations,nfit(0)
      end if  ! lq.lt.3

      chq2=0.0
      chq =0.0
      do moca=1,iprobe
       do k=1,nfit(0)
        ks=nfit(k)
        if(b_datset(ks))then
         j1=1
         call frametheory(ks,'test',j1,info)
c         write(6,'(10f12.3)')(thfk(i),i=1,256,16)
         if(info.gt.0) goto 10
         nchansp=cdata(ks,ip_chan(ks))
         if(lq.eq.2) then
          do i=1,nchansp
           r=ran3(idumran)
           do while(r.lt.0.000001)
            r=ran3(idumran)
           end do
           y=sqrt(-thfk(i)*alog(r))
           if(ran3(idumran).gt.0.5) y=-y
           real_fr(mfpt+i)=int(thfk(i)+y)
          end do
         else if(lq.eq.1)then
          do i=1,nchansp  
           y=real_fr(mfpt+i)-thfk(i)
           if(ran3(idumran).gt.0.5) y=-y
           real_fr(mfpt+i)=int(thfk(i)+y)
          end do
         else if(lq.eq.3)then
c         true Monte Carlo ------------------------
          nthar=abs(ktheo(ks,0,0))
          nth=ktheo(ks,0,1)
          lth=ktheo(ks,0,2)
          call getname(0,nth,lth,property,name_theo)
          mosstrans=index(name_theo,'Mossbauer transmission').gt.0
          mossreem=index(name_theo,'Mossbauer reemission').gt.0
c         xxx=index(name_theo,'name_xxx').gt.0
          if(nthar.gt.3) then
           do klev=3,nthar,2
            nth=ktheo(ks,0,klev)
            lth=ktheo(ks,0,klev+1)
            call getname(1,nth,lth,property,name_theo)
            if(index(name_theo,'Mossbauer transmission').gt.0)
     &                                              mosstrans=.true.
            if(index(name_theo,'Mossbauer reemission').gt.0)
     &                                              mossreem=.true.
           end do
          end if

          info=1
          if(mosstrans) then
           info=0
           call mosstrans_mcarlo(nchansp)
           do i=1,nchansp
            real_fr(mfpt+i)=int(dfdp(i)+0.5)
            real_fr(mdfpt+i)=sqrt(abs(real_fr(mfpt+i)))
           end do
          else if(mossreem) then
           info=2
          end if 

          if(info.gt.0) then
           infotext(1)='effi/simulation/montecarlo'
           infotext(2)='no simulation'
           infotext(3)='Monte Carlo routine not yet implemented for'
           infotext(4)=name_theo
           infotext(5)=' '       
          end if        
          goto 10 
         end if
        end if
       end do

       mff=mf
       do i=1,mf
         kp(ifree(i))=kpfr(i)
       end do
c       write(6,'(a,10i4)')' fit mc kpfr=',(kpfr(i),i=1,mf)

       if((mf.eq.0.or.mf.gt.maxpf).and.chft.ne.'test') then
        info=1
        infotext(1)='effi/mcarlo/fit/mf_0'
        infotext(2)=
     &    '**error** no free or number of fit parameter too large'
        infotext(3)=
     &    'the command {te} is executed if iterations is set to 1'
        infotext(4)=' '
        goto 10
       end if

       if(iprobe.eq.1.or.iterations.le.1) then
        chft='test' 
        do i=0,nfit(0)
         ntest(i)=nfit(i) 
        end do
       end if

       path2=0.0
       iter_count=1

c        write(6,'(a,i4,3g15.5)')' iterations, pathw,convcrit=',
c     &                            iterations,pathw,convcrit
       ochisq=1.0e10
       do while(iter_count.le.iterations)

        do k=1,mp
         pold(k)=p(k)
        end do
        call  fit(chft,mf,iter_count,pathw,
     &                    ochisq,chisq,chi_p,path1,path2,info)
        if(info.gt.0) goto 10

        if(iter_count.gt.0) then
         iter_count=iter_count+1
         dchi=ochisq-chisq
         if(dchi.gt.convcrit) then
          ochisq=chisq
         else
          goto 5
         end if
        else
         iter_count=-iter_count+1
        end if        
c        write(6,'(a,2i4,3g15.5)')'fit mc mf,iter_count dchi,chi_sq=',
c     &                            mf,iter_count,dchi,chisq
       end do  ! iter_count
 5     continue
       chq=chq+ ochisq
       chi_ar(moca)=ochisq
       chq2=chq2+ ochisq**2
      end do   ! 1-iprobe

      if(iprobe.eq.1.and.iterations.le.1)write(nwr,'(a)')
     & ' chi_square value of generated spectrum:'
 
      if(iterations.gt.1.and.iprobe.gt.1) write(nwr,'(/,a)')
     &' Monte Carlo generated spectra are fitted with routine '//chft
      if(iprobe.gt.1)then
       write(nwr,'(a)')' chi_square values of generated specta:'     
       ipl=npl/8
       i10=ipl/10
       i1=ipl-i10*10+i0
       ch2=char(i10+i0)//char(i1)
       ch8='('//ch2//'f8.4)'
       if(iprobe.le.8)then
        write(nwr,ch8)(chi_ar(i),i=1,iprobe)
       else
        write(nwr,ch8)(chi_ar(i),i=1,8)
        write(nwr,ch8)(chi_ar(i),i=9,iprobe)
       end if
       write(nwr,'(a,f10.4,a,f10.4)')'      <chi_square>=',chq/iprobe,
     &                   ' +-',sqrt(abs(chq2-chq**2/iprobe)/iprobe)
      else
       if(iprobe.eq.1)write(nwr,'(a,f10.4)')'     chi_square=',chisq
      end if
 10   return
      end


      subroutine dp_sigma(info)
      implicit none  
      include 'comglb.for'
      include 'compar.for'
      include 'comfrm.for'
      include 'comdat.for'
      include 'comdas.for'
      include 'comthe.for'
      include 'comths.for'

      integer info,i0,mf,mchan,k,km1_mchan,ks,j,j1,idumscrt,idumsc,
     &        kpa,nf,nchansp,jj,i,i1,i10,l,l1,ip
      real    theo_0,z,zs,zchisq,h,zz,ran3,r,y,s,chisq
      character formpr*14,ch2*2,chflen*256,ch1*1
      parameter(idumscrt=maxscrt-max8*8-maxpf-11-64)
      common/scrt/theo_0(max8*8),zz(maxpf),formpr(3),ch2,chflen,
     &            idumsc(idumscrt)

      integer   ib_df,kana,ibmisfit
      real      thfk,dfdp,ddpk
      common/fr2fi/thfk(max8),dfdp(max8),ddpk,ib_df,kana,ibmisfit

      ch2='24'
      formpr(1)='2a1,i1,3x,a'//ch2//','
      formpr(2)='2a1,i2,2x,a'//ch2//','
      formpr(3)='2a1,i3,1x,a'//ch2//','

       i0=ichar('0')
       mf=0
       do kpa=1,mp
        if (kp(kpa).eq.1) then
         mf=mf+1
         kf(mf)=kpa
        end if
       end do

       mchan=ichanks(ichanks(0))
       nf=0    
       do k=1,nfit(0)
        km1_mchan=(k-1)*mchan
        z=0.0
        ks=nfit(k)
        if(b_datset(ks))then
         j1=1
         call calpl('free')
         call frametheory(ks,'test',j1,info)
         if(info.gt.0) goto 10

         do j=2,ichanks(0),2
          do i=ichanks(j-1),ichanks(j)

           r=(abs(ran3(iseed))-0.5)*.99+0.5
           y=sqrt(abs(real_fr(mdfpt+i)**2*alog(r)))
           s=ran3(iseed)
           if(s.gt.0.5) y=-y
           theo_0(km1_mchan+i)=thfk(i)+y
           z=z+(y/real_fr(mdfpt+i))**2
           nf=nf+1

          end do
         end do
c         write(6,'(a,f15.2)')'z=',z
        end if
        zz(k)=z
       end do


       chisq=0.0    
       zs=0
       do j=1,mf
        kpa=kf(j)
        do k=1,nfit(0)
         km1_mchan=(k-1)*mchan
         z=0
         ks=nfit(k)
         if(b_datset(ks))then
          p(kpa)=p(kpa)+ddp(kpa)
          j1=1
          call calpl('free')
          call frametheory(ks,'test',j1,info)
          p(kpa)=p(kpa)-ddp(kpa)
          if(info.gt.0) goto 10
          nchansp=cdata(ks,ip_chan(ks))
          do jj=2,ichanks(0),2
           do i=ichanks(jj-1),ichanks(jj)
            z=z+((theo_0(km1_mchan+i)-thfk(i))/real_fr(mdfpt+i))**2
           end do
          end do
         end if        
c         write(6,'(a,2f15.3)')' fit  z,zz(k)=',z,zz(k)
         if(abs(z-zz(k)).gt.z*0.00001) then
          zs=zs+z
          chisq=chisq+zz(k)
         end if
        end do 

c        write(6,'(a,2f15.3)')' fit  zs,chisq=',zs,chisq
        if(zs.gt.0.0.and.chisq.gt.0.0) then 
         zchisq=zs-chisq
         h=sqrt(chisq/abs(zchisq)*0.66)
          ddp(kpa)=h*ddp(kpa)
         i10=1/h
         l=0
         do while(i10.gt.1)
          i10=i10/10
          l=l+1
         end do        
         l1=ichar(np(kpa)(59:59))+l
         if(l1-i0.gt.9) then
          np(kpa)(55:60)='e08.2)'
         else
          np(kpa)(59:59)=char(l1)
          l1=10*(ichar(np(kpa)(56:56))-i0)+ichar(np(kpa)(57:57))-i0+l
          i10=l1/10
          i1=l1-10*i10
          np(kpa)(56:57)=char(i10+i0)//char(i1+i0)
         end if
         ch1=' '
          k=kpa/100
          ip=3
          if(k.eq.0) then
          ip=2
          k=kpa/10
          end if
          if(k.eq.0) ip=1
          chflen='('//formpr(ip)//np(kpa)(25:60)
c          write(6,'(a)')chflen
          write(nwr,chflen)
     &    ch1,'p',kpa,np(kpa),p(kpa),up(kpa),op(kpa),ddp(kpa)
        else
         ch1='o'
          k=kpa/100
          ip=3
          if(k.eq.0) then
         ip=2
         k=kpa/10
         end if
         if(k.eq.0) ip=1
          chflen='('//formpr(ip)//np(kpa)(25:60)
c          write(6,'(a)')chflen
          write(nwr,chflen)
     &    ch1,'p',kpa,np(kpa),p(kpa),up(kpa),op(kpa),ddp(kpa)
        end if
       end do
 10    continue

       return
       end 

      subroutine dp_pm(info)  
      implicit none
      include 'comglb.for'
      include 'compar.for'
      include 'comfrm.for'
      include 'comdat.for'
      include 'comths.for'

      integer info,mf,kpa,j,k,ks
       real    s,ran3

       mf=0
       do kpa=1,mp
        if (kp(kpa).eq.1) then
         mf=mf+1
         kf(mf)=kpa
        end if
       end do

       do j=1,mf
        kpa=kf(j)
        do k=1,nfit(0) 
         ks=nfit(k)
         if(b_datset(ks))then
          s=ran3(iseed)
          if(s.gt.0.5) then
           p(kpa)=p(kpa)+ddp(kpa)
          else
           p(kpa)=p(kpa)-ddp(kpa)
          end if
         end if
        end do 
       end do
       call calpl('free')

       return
       end 

      subroutine mosstrans_mcarlo(nu_channel) 
      implicit none
      include 'comglb.for'
      include 'compar.for'
      include 'comfrm.for'
      include 'comscd.for'

      integer   ib_df,kana,ibmisfit
      real      thfk,dfdp,ddpk,
     &          avsp,f_factor,base,bg_fraction,dumvsp
      common/avstrf/avsp,f_factor,base,bg_fraction,dumvsp(ivstrf-4)
      common/fr2fi/thfk(max8),dfdp(max8),ddpk,ib_df,kana,ibmisfit

      integer i,nu_channel,info,ledit
      real win_fraction,countrate,deadtime,
     &     sweepfrequency,ameasuretime,printtime,dwelltime,start,finish,
     &     bg_fract,fres
      character chcrd*132,chcom*132

        bg_fract=bg_fraction+(1.-f_factor)*(1.-bg_fraction)
        fres=f_factor*(1.-bg_fraction)
        do i=1,nu_channel
         thfk(i)=1.+(thfk(i)/base-1.)/fres
        end do

c       25:32 information about the helpfile (th110105)
c             if 25:32 is empty chcom is taken by subroutine conread
c             as defined below
c       33:37 format (effi specific) of the number
c                  1234567890123456789012345678901234567
        nconwr(1)='win_fraction                    f04.2)'
        nconwr(2)='countrate(/sec)                 e08.1)'
        nconwr(3)='deadtime (ms)                   e06.1)'
        nconwr(4)='sweepfrequency(Hz)              f05.1)'
        nconwr(5)='measure_time (sec)              i03)'
        nconwr(6)='printtime (sec)                 i03)'
        nconwr(7)='iseed                           i09)' 
        
        win_fraction=0.8     ! fraction of conuts in the window 
        countrate=1.e5       ! in the window  
        deadtime=1.e-5    
        sweepfrequency=10.
        ameasuretime=30   
        printtime=10      
        iseed=222 
        
        chcrd='1-7'
        chcom='<Mossbauer_spectrum>/trans/simulation/montecarlo'

        conwr(1)=win_fraction    ! fraction of conuts in the window
        conwr(2)=countrate       ! in the window
        conwr(3)=deadtime    
        conwr(4)=sweepfrequency
        conwr(5)=ameasuretime   
        conwr(6)=printtime      
        conwr(7)=iseed         

        ledit=0
        call conread(chcrd,chcom,ledit)

        win_fraction=  conwr(1)
        countrate=     conwr(2)
        deadtime=      conwr(3)
        sweepfrequency=conwr(4)
        ameasuretime=  conwr(5)
        printtime=     conwr(6)
        iseed=         conwr(7)
                  
        dwelltime=1./sweepfrequency/nu_channel
        call cpu_time(start)
        call mosstranscounts(base,nu_channel,win_fraction,bg_fract, 
     &      countrate,deadtime,dwelltime,ameasuretime,printtime,iseed) 

        call cpu_time(finish)
        print '(" runtime = ",f15.3," seconds.")',finish-start

        return
        end
