       subroutine  n_optics(maxl,NL,b_Cm,MAXGRP,GEOSTR,SQ,Hextz,Cmatrix,
     &                              VKD,roughlay,b_up,b_down,Rmatrix)


C*******************************************************************    
C                               OPTICS
C
C          Reflectivity calculation of anizotropic multilayers
C          in the Neutron case. 
C
C          (method of G. H. Borzbov et. al. - L. Deak et. al.)
C
C                         Laszlo Deak, Hartmut Spiering 
C
C                         Budapest, 2000. December
C				
C						Khaidukov Yuri
C
C                         Budapest, 2004, September
C ***************************************************************** 
C
C Geometry:
C
C
C               incident        reflected                    
C                          
C                        \    /    
C                         \  /
C                      FI (\/
C      Z=0>           -------------  
C           covering  _____________         [D(1),K(1,I,J)]  
C                                  >Layer#1 [D(2),K(2,I,J)]
C                 1.  .............     
C                           .
C                           .
C                     .............
C Z=NP*Sum(D(L)) NL.  _____________Layer#NL [D(NL-1),K(NL-1,I,J)]
C       I              
C          substrate  _____________         [D(NL),K(NL,I,J)]
C                            `                 ^
C                              `            infinit or finit
C                             refracted
C
C 
C  Variables of system:
C
C  INPUT:
C
C                           covering   layer #L     substrate
C
C            thickneses:         D(1)     D(L)        D(NL)
C
C        susceptibility:
C          (nuclear+magnetic)
C                                K(1)     K(L)        K(NL)
C
C         where  L=1..NL  I,J=1,2,3 
C
C  OUTPUT:
C
C         The 2x2 complex reflectivity (or transmitivity) matrix:  R
C
C ******************************************************************
C       
C
       implicit none
C      input:
       integer    NL,NLOG
C      NL: number of layers  (in period)
C      output:
       complex*16 R(2,2)
C      other variables:
       complex*16 C0,C1,CI,M(4,4),ML(4,4),MS(4,4),KL(2,2),
     &            Gamma(2,2),Q
       real*8     DL,Hext4,QP,QD
       integer    I,J,L,n_rep,maxl
C
       parameter(C0=(0.D0,0.D0), C1=(1.D0,0.D0), CI=(0.D0,1.D0))
C**************************************
c
       complex*8 Cmatrix(2,2,maxl),Rmatrix(2,2)
       real*4    VKD(maxl),SQ,roughlay(maxl),Hextz
       logical b_Cm(maxl),b_odz,b_first,b_up,b_down
!       dimension laystr(2*maxl)
C New variabes has been added due do the concept of GROUPS.
C The structure of the subroutine now familiar to g_optics.f (Khaidukov)
	integer MAXGRP,GEOSTR(0:2*maxl),GRP,LMIN,LMAX,Lprev,POINTER,Lnext
	logical b_grp,prev_dgnl,both_dgnl,both_NONdgnl
	real*4 absCmatr
	complex*16 KLprev(2,2),UnMat(2,2)
	real*8 DabsMatr 
C**************************************
C	NL=5 !Unused variable byt I must put it for EFFI

      UnMat(1,1)=C1
	UnMat(1,2)=C0	 !Unit matrix
	UnMat(2,1)=C0
	UnMat(2,2)=C1

C
       QD=dble(SQ)
       Q =cmplx(QD,0.D0)       

       Gamma(1,1)=C1
       Gamma(1,2)=C0
       Gamma(2,1)=C0
       Gamma(2,2)=C1 ! no analizator
C
       if(b_up)then

         Hext4=4.D0*dble(Hextz)
         QP=QD*QD-Hext4
         if(QP.ge.0.D0)then
           Q= cmplx(sqrt(QP),0.D0)
C           write(6,'(2E16.3)')Q
         else
           Q= cmplx(0.D0,-sqrt(-QP))
C           write(6,'(2E16.3)')Q
         endif

CCC         if(QP.lt.1.D0)then
CCC           Gamma(1,1)= sqrt(C1+cmplx(QP,0.D0))
CCC           Gamma(2,2)= sqrt(C1-cmplx(QP,0.D0))
CCC           if(QP.lt.-1.D0)then
CCC             Gamma(1,1)=-sqrt(C1+cmplx(QP,0.D0))
C             Gamma(2,2)= sqrt(C1-cmplx(QP,0.D0))
CCC           endif

CCC         else
CCC           Gamma(1,1)= sqrt(C1+cmplx(QP,0.D0))
CCC           Gamma(2,2)=-sqrt(C1-cmplx(QP,0.D0))
CCC         endif
C
C         Gamma(1,1)= QD/Q

         if(QP-Hext4.ge.0.D0)then
           Gamma(1,1)=cmplx(QD/dble(Q),0.D0)
           Gamma(2,2)=cmplx(sqrt(QP-Hext4)/dble(Q),0.D0)
         else
           if(QP.ge.0.D0)then
             Gamma(1,1)=cmplx(QD/dble(Q),0.D0)
             Gamma(2,2)=cmplx(0.D0,-sqrt(Hext4-QP)/dble(Q))
           else
             Gamma(1,1)=cmplx(0.D0,-QD/dimag(Q))
             Gamma(2,2)=cmplx(-sqrt(Hext4-QP)/dimag(Q),0.D0)
           endif
         endif
C         write(6,*)SQ,Gamma(1,1)*Gamma(2,2)
         Gamma(1,2)=C0
         Gamma(2,1)=C0
C
       endif         ! we measure the '+' beam
C
       if(b_down)then

         Hext4=4.D0*dble(Hextz)
         QP=Hext4/(QD*QD+Hext4)
         Q=sqrt(cmplx(QD*QD+Hext4,0.D0))
         Gamma(1,1)=sqrt(C1+cmplx(QP,0.D0))
         Gamma(1,2)=C0
         Gamma(2,1)=C0
         Gamma(2,2)=sqrt(C1-cmplx(QP,0.D0))
c         write(6,*)SQ,Gamma(2,2),Q

       endif         ! we measure the '-' beam
C
!***************    This block was of the previous structure***************
!****** Commented by Khaidukov ********
C
C       if(NL.gt.2)then
C        b_first=.true.
C        L=2
C 100    continue
C        do I=1,4
C         do J=1,4
C          ML(I,J)=C0
C         end do
C        end do
C        do I=1,4
C         ML(I,I)=C1
C        end do
CC
C        if(L.gt.NL-1) goto 110
C        n_rep=laystr(2*L-1)
C        igrp=laystr(2*L)
C          Lbegin=L                                 ! Hartmut#1
C        do while(laystr(2*L).eq.igrp)	  
CC        write(6,'(a,i4,f10.4)')' lopp L,VKD(L)=',L,VKD(L)  
CC        Calculation of 4x4 integral propagation matrix for layer#I:
C
C         if(VKD(L).gt.0.) then
C
C          b_odz=.not.b_Cm(L)
C
C
C          KL(1,1)=Cmatrix(1,1,L)
C          KL(1,2)=Cmatrix(1,2,L)
C          KL(2,1)=Cmatrix(2,1,L)
C          KL(2,2)=Cmatrix(2,2,L)
C
C          DL=VKD(L)
C          call nfill(b_odz,Q,KL,DL,FL,XL)
C
C             if(laystr(2*L+2).eq.igrp) then  !
C                Lnext=L+1                    !
C             else                            ! Hartmut#1
C                Lnext=Lbegin                 !
C             end if                          !
C
C           
C          call genM(b_odz,KL,FL,XL,roughlay(L),roughlay(L+1),M)
C          call genM(b_odz,KL,FL,XL,roughlay(L),roughlay(Lnext),M) !Hartmut#1
C
C          call mat4(M,ML,ML) 
C
C         endif ! VKD(L) > 0.
C         L=L+1
C        end do ! while  group matrix
C
CC     For periodic multilayers the int. propagation matrix is (ML)**n_rep, 
CC     where n_rep is the number of periods.
CC
CC        
C         if(n_rep.gt.1)then
Cc          write(6,'(a,i4)')' n_rep=',n_rep
C          do I=1,4
C           do J=1,4
C            M(I,J)=ML(I,J)
C           end do
C          end do
Cc        
C          NLOG=alog(float(n_rep))/alog(2.0)
C
C          do I=1,NLOG
C           call mat4(ML,ML,ML)
C          end do
Cc
C          do I=1,n_rep-2**NLOG
C           call mat4(M,ML,ML)
C          end do
CC
C         end if  ! n_rep>1
CC
C         if(b_first) then
C          b_first=.false.
C          do I=1,4
C           do J=1,4
C            MS(I,J)=ML(I,J)
C           end do
C          end do
C         else
C          call mat4(ML,MS,MS)
C         end if         
C        goto 100
C      
C 110    continue
C
C        end if  ! NL>2
C
CC
CC     Calculation of coating layer:
C
C        L=1
C	if(VKD(L).gt.0.)then
C
C         b_odz=.not.b_Cm(L)
C
C         KL(1,1)=Cmatrix(1,1,L)
C         KL(1,2)=Cmatrix(1,2,L)
C         KL(2,1)=Cmatrix(2,1,L)
C         KL(2,2)=Cmatrix(2,2,L)
C
C         DL=VKD(L)
C         call nfill(b_odz,Q,KL,DL,FL,XL)
C         
C         call genM(b_odz,KL,FL,XL,roughlay(L),roughlay(L+1),M)
C	 call mat4(MS,M,MS)
C	endif
C
CC     Calculation of the substrat "layer":
C        DL=VKD(NL)
C        if(VKD(NL).lt.0.)DL=1.D9  
C        L=NL
C	if(DL.gt.0.d0)then
C
C         b_odz=.not.b_Cm(L)
C
C          KL(1,1)=Cmatrix(1,1,L)
C          KL(1,2)=Cmatrix(1,2,L)
C          KL(2,1)=Cmatrix(2,1,L)
C          KL(2,2)=Cmatrix(2,2,L)
C
C         call nfill(b_odz,Q,KL,DL,FL,XL)
C            
C         call genM(b_odz,KL,FL,XL,roughlay(L),0.0,M)
C         call mat4(M,MS,MS)
C	endif
CC
!**************** End of the block of the previous structure************

!**************New block of the GROUP concept**********
!*********Changes by Khaidukov************

       b_first     =.true.
       POINTER=0                                     
       do GRP=1,MAXGRP      ! loop for the groups
C       To set the characteristic matrix to unit matrix
        do I=1,4
         do J=1,4
          ML(I,J)=C0
         end do
         ML(I,I)=C1
        end do
        b_grp=.false.
C
        LMIN =POINTER+1
        LMAX =POINTER+GEOSTR(2*GRP)
        n_rep=GEOSTR(2*GRP-1)
C
        do L=LMIN,LMAX

	  if(VKD(L).gt.0.) then
C          b_odz=.false.                              
	    b_odz=.not.b_Cm(L)
		b_grp=.true.  			 
            DL=VKD(L)
            KL(1,1)=Cmatrix(1,1,L)
            KL(1,2)=Cmatrix(1,2,L)
            KL(2,1)=Cmatrix(2,1,L)
            KL(2,2)=Cmatrix(2,2,L)
            absCmatr=real(Cmatrix(1,2,L)*conjg(Cmatrix(1,2,L))+
     &                    Cmatrix(2,1,L)*conjg(Cmatrix(2,1,L)))
           if(absCmatr.lt.1.e-15) then
		 b_odz=.true.
	     end if
C    		 
C          if(L.eq.LMAX.and.n_rep.GT.1) then  !
C           Lnext=LMIN         ! For peiodical structure the roughness
C          else                ! is also periodical! The next layer to
C           Lnext=L+1          ! the last element is the first layer! 
C           if(Lnext.gt.NL) Lnext=NL
C          end if              

          call genM(b_odz,QD*0.5D0,DL,KL,M)
C************* Roughness************
	IF (roughlay(L).GT.0) THEN
           if (n_rep.GT.1.and.L.eq.Lmin) then  ! In case of repeating layers
	     Lprev=Lmax	                       ! we are more interested in the 
	     else                              !information about roughnesses
                              		       !in repeated ones. 
	     Lprev=L-1			               !#Khaidukov
	     END IF

          if(n_rep.GT.1.and.L.eq.LMAX) then  !
           Lnext=LMIN         ! For peiodical structure the rougness
          else                ! is also periodical! The next layer to 
           Lnext=L+1          ! the last element is the first layer! 
           if(Lnext.gt.NL) Lnext=NL
          end if              


	    DO I=1,2
	     DO J=1,2 
	      IF (Lprev.EQ.0) THEN 
	       KLprev(I,J)=C0     ! In case of 1st layer KLprev is for vacuum
	       prev_dgnl=.true.
		   END IF                
		  IF (Lprev.GT.0) THEN 	  
          	   KLprev(I,J)=Cmatrix(I,J,Lprev)
	           prev_dgnl=.not.b_Cm(Lprev)							
	       ENDIF

		 END DO
	    END DO

      DabsMatr=dreal(KLprev(1,2)-KL(1,2))*conjg((KLprev(1,2)-KL(1,2)))+
     &              (KLprev(2,1)-KL(2,1))*conjg((KLprev(2,1)-KL(2,1)))         
	
      IF (prev_dgnl.and.b_odz) then	  !
	both_dgnl=.true.				  !	 Rohlsberger roughness1
	both_NONdgnl=.false.              !
	ELSEIF (DabsMatr.lt.1.e-15)	then  !
	both_dgnl=.true.				  !  Rohlsberger roughness1
	both_NONdgnl=.false.			  !
	ELSEIF (.not.prev_dgnl.and..not.b_odz) then
	both_dgnl=.false. 				  !
      both_NONdgnl=.true.				  !Deak's roughness
	ELSE							  
	both_dgnl=.false. 				  !	Rohlsberger roughness2
      both_NONdgnl=.false.			  !
	ENDIF

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C	both_dgnl=.false. 				  !DONT FORGET TO REMOVE!!!!!!!!!
C      both_NONdgnl=.TRUE.				  !Deak's roughness
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      call roughness(both_dgnl,both_NONdgnl,roughlay(L),roughlay(Lnext),
     &QD*0.5D0,KLprev,KL,M)
		END IF !Roughness

          call mat4(M,ML,ML) 
         endif ! VKD(L) > 0.         
         end do ! L

C     For periodic multilayers the int. propagation matrix is (ML)**n_rep, 
C     where n_rep is the number of periods.
C
C        
        if(n_rep.gt.1.and.b_grp)then
         do I=1,4
          do J=1,4
           M(I,J)=ML(I,J)
          end do
         end do
C        
         NLOG=alog(float(n_rep))/alog(2.0)
C
         do I=1,NLOG
          call mat4(ML,ML,ML)
         end do
C
         do I=1,n_rep-2**NLOG
          call mat4(M,ML,ML)
         end do
C
        end if  ! n_rep>1

        if(b_first) then
         b_first=.false.
         do I=1,4
          do J=1,4
           MS(I,J)=ML(I,J)
          end do
         end do
        else
         call mat4(ML,MS,MS)
        end if         
C
        POINTER=POINTER+GEOSTR(2*GRP)
       end do ! GRP

C     Now we have the integreted propagation matrix of the structure: ML
C     Calculation of reflectivity matrix (2x2):
C
	call reflG(Gamma,MS,R)
C	call refl(MS,R)
C
        Rmatrix(1,1)=R(1,1)
        Rmatrix(1,2)=R(1,2)
        Rmatrix(2,1)=R(2,1)
        Rmatrix(2,2)=R(2,2)
C
c	if(abs(R(2,2)**2).gt.1.)
c     &    write(6,*) 'SQ,abs(R(2,2)) ', SQ,abs(R(2,2))

C       write(6,'(8e15.4)')Rmatrix

	return
	end

C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
	subroutine reflG(G,M,R)
C
C *******************************************************************
C
C                 Calculation of reflectivity matrix (2X2)
C
C ******************************************************************
C
      implicit none
C
	complex*16 M(4,4),R(2,2),AA,BB,CC,DD
	complex*16 R1(2,2),R2(2,2),R11(2,2),G(2,2)
	integer    I,J
C
C    Definition of 2x2 block matrices:
C
C	do I=1,2
C	  do J=1,2
C	    AA= M(I,J)
C  	    BB= M(I,J+2)
C	    CC= M(I+2,J)
C	    DD= M(I+2,J+2)
C           R1(I,J)= AA-BB-CC+DD
C           R2(I,J)= AA+BB-CC-DD
C	  end do
C	end do
	do I=1,2
	  do J=1,2
	    AA= M(I,J)
  	    BB= M(I,J+2)
	    CC= M(I+2,J)
	    DD= M(I+2,J+2)
            R1(I,J)= AA-CC
            R2(I,J)= DD-BB
	  end do
	end do
C
	call mat2(R1,G,R11)
C
	do I=1,2
	  do J=1,2
            R1(I,J)= R11(I,J)+R2(I,J)
            R2(I,J)= R11(I,J)-R2(I,J)
	  end do
	end do
C
	call inv2(R1,R11)
C
	call mat2(R11,R2,R)
C
	return
	end

































